;+
; Mission: ASO-S/HXI
; Purpose:
;  detector choice
; Inputs:
;  none   -- default, all detectors used
;  inarr  -- select detectors
; Keywords:
;  id   -- Name of detectors
;  type -- Type of detectors
;  hlp  -- help
; Outputs:
;  detectors number of user selected
;
; History:
;  Weichen, 2022/03/03
;  Zhentong Li, 2023/11/04, detype 'back' -> 'bkg'

FUNCTION hxi_det_config, inarr,id=id,type=type, hlp=hlp

 if  keyword_set(hlp) then begin
  print, 'dets = hxi_det_config(inarr), '
  print, 'inarr can use strings from bellow: '
  print, 't1 t2 t3 b1 b2 b3 b4 b5 '
  print, 't1 and t2 means totalflux for full open detector with normal Al window, '
  print, 't3 represent total flux for full open detector with thin Al window; '
  print, 'b1 to b5 means the background detector. '
  print, 'OR '
  print, 'the elements of inarr can use number 0 or 1,'
  print, 'e.g.  inarr = intarr(8)+1 '
  print, '      inarr = [1,1,0,1,1,0,0,1] '
 endif

  det_id= ['t1','t2','t3','b1','b2','b3','b4','b5']
  default,inarr,  det_id
  det = intarr(8)
  
if datatype(inarr) EQ 'STR' then begin
  ind = where_arr(det_id,strlowcase(inarr), nind)
    if nind NE n_elements(inarr) then begin
      message,'ERROR - Unrecognized detector of'+string(inarr),/con
      return, -1
    endif
  
  det[ind] =1
endif else begin
  if n_elements(inarr) NE 8 then $
     message, 'inarr with number must have 8 elements for full open and back detector !'
    det = 0> inarr <1
  if total(inarr) EQ 0 then $
     message, 'inarr must be =1 for at least one detector.'

endelse

if keyword_set(id) then det=det_id(where(det eq 1))

detype= ['thick','thin','bkg']
if keyword_set(type) then begin
  det=hxi_det_config(inarr)
  tp =[det(0)+det(1), det(2), total(det(3:7))] 
  itp =where(tp GE 1)
  det = detype(itp)
endif

RETURN, det

END
