;+
; Mission     : ASO-S/HXI
; Name        : hxi_get_config_data
; Purpose     : get HXI configuration data of sub-collimators. 
;         maybe can use the configuration file, rather than the code here
; Category    : imaging
; Inputs      :
; Outputs     :
;2018/03/27 by yplee
;2018/10/21 by Yang Su, pitch0angle0, ideal grids; modified the code to make c_index, s_index earier to get.
;2020/01/18 by Yang Su, beta version after a series of test versions. added step,r0, atype.
;                        used final configration for pitches, added choice of angles.
;                         changed most of the names of the vairables.
;                         added grid thickness.
;2020/07/04 by Yang Su, added gtypes.

function hxi_get_config_data, gtype=gtype
 
  default, gtype, 1   ;type of configration for positioning angles
 
  pitches= [36.,52.,76.,108.,156.,224.,344.,524.,800.,1224.]    ;um
  thickness=  [1.0,1.4,1.7, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0]  ;mm
  slit_width =  [20,31, 46, 54, 78, 112, 172, 262, 400, 612]   ;um
 
  subcoll_L = 119.01038   ;cm
 
  grid_p=[fltarr(8) +pitches[0],$
          fltarr(10)+pitches[1],$
          fltarr(10)+pitches[2],$
          fltarr(10)+pitches[3],$
          fltarr(10)+pitches[4],$
          fltarr(10)+pitches[5],$
          fltarr(10)+pitches[6],$
          fltarr(10)+pitches[7],$
          fltarr(6) +pitches[8],$
          fltarr(7) +pitches[9]]

  grid_slit=[fltarr(8) +slit_width[0],$
          fltarr(10)+slit_width[1],$
          fltarr(10)+slit_width[2],$
          fltarr(10)+slit_width[3],$
          fltarr(10)+slit_width[4],$
          fltarr(10)+slit_width[5],$
          fltarr(10)+slit_width[6],$
          fltarr(10)+slit_width[7],$
          fltarr(6) +slit_width[8],$
          fltarr(7) +slit_width[9]] 
          
  grid_t=[fltarr(8) +thickness[0],$
          fltarr(10)+thickness[1],$
          fltarr(10)+thickness[2],$
          fltarr(10)+thickness[3],$
          fltarr(10)+thickness[4],$
          fltarr(10)+thickness[5],$
          fltarr(10)+thickness[6],$
          fltarr(10)+thickness[7],$
          fltarr(6) +thickness[8],$
          fltarr(7) +thickness[9]]
 
  grid_phases=[fltarr(8) ,$
          fltarr(10) ,$
          fltarr(10) ,$
          fltarr(10) ,$
          fltarr(10) ,$
          fltarr(10) ,$
          fltarr(10) ,$
          fltarr(10) ,$
          fltarr(6) ,$
          fltarr(7) ]
 
  c_ang=float([ 25, 70, 115 , 160,  $
                5,  41, 77,  113, 149,   $
                32, 68, 104, 139, 176,   $
                23, 59, 95,  131, 167,  $
                14, 50, 86,  122, 158,   $
                5,  41, 77,  113, 149,   $
                23, 59, 95,  131, 167,  $
                5,  41, 77,  113, 149,   $
                23, 83, 143, $
                53, 173, 113 ])
                
         
  grid_angles= fltarr(n_elements(c_ang) *2 +1 )
  grid_ph= fltarr(n_elements(c_ang) *2 +1 )
  
  grid_c_index=indgen(n_elements(c_ang))*2
  grid_s_index=indgen(n_elements(c_ang))*2+1
  
  grid_angles[grid_c_index]=c_ang
  grid_angles[grid_s_index]=c_ang
  grid_angles[-1]= 113.
  
  grid_ph[grid_c_index]=0.
  grid_ph[grid_s_index]=90.
  grid_ph[-3]= 0.
  grid_ph[-2]= 120.
  grid_ph[-1]= 240.
  
  grid_rad_f= 36.*0.1/2.  ;sqrt(36./!pi)  ;cm
  grid_rad_r= 22.*0.1/2.   ;sqrt(22./!pi)  ;cm
 
  pitches= [36.,52.,76.,108.,156.,224.,344.,524.,800.,1224.]    ;um
  thickness=  [1.0,1.4,1.7, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0]  ;mm
  slit_width =  [20,31, 46, 54, 78, 112, 172, 262, 400, 612]   ;um

  readme=[  'ASO-S/HXI grid parameters: ',$
            'grid_p        :   grid pitch in um',$
            'grid_slitw    :   grid slit width in um',$
            'grid_angles   :   grid position angles in degree',$
            'grid_c_index  :   cos-type index',$
            'grid_s_index  :   sin-type index',$
            'grid_t        :   grid thickness in mm',$
            'grid_phases   :   grid phases in degree',$
            'grid_rad_f    :   radius of front grids in cm',$
            'grid_rad_r    :   radius of rear grids in cm',$
            'subcoll_L     :   length of subcollimators in cm']
    
  grid_conf={ grid_p        :   grid_p          ,$
              grid_slitw    :   grid_slit       ,$
              grid_angles   :   grid_angles     ,$
              grid_c_index  :   grid_c_index    ,$
              grid_s_index  :   grid_s_index    ,$
              grid_t        :   grid_t          ,$
              grid_phases   :   grid_ph         ,$
              grid_rad_f    :   grid_rad_f      ,$
              grid_rad_r    :   grid_rad_r      ,$
              subcoll_L     :   subcoll_L}
 
    ;36.  52  75  107 155 223 341 522 799 1222
    ;[36,52,76,108,156,224,344,524,800,1224.]

  return,grid_conf


end