;+
; Mission  : ASO-S/HXI
;
; NAME:
;   HXI_PHOTO_DIS
;
; Puropse:
;   Returns the spatial distribution of incident photons, stored in an 11*9 array 
;
; INPUT: 
;   det_counts_arr - float[91], counts of modu-dets
; 
; OUTPUT:
;   counts_dis  - float[11,9,2], index 0 of the third dimension is counts of different dets
;                 index 1 is the corresponding detector number
; 
; NOTE: 
;   The array are sorted according to the detector distribution (refer to hxi_get_grid_info) 
;   Det.92-99 are spec-dets, here set their counts eq 0
;
; History:
;     Written by Chang-xue Chen, 2023/06/13

function hxi_photo_dis, det_counts_arr
  d = det_counts_arr
  if n_elements(d) eq 91 then d = [d,0,0,0,0,0,0,0,0]
  
  counts_dis = make_array(11,9,2,/float)
  ;line1 
  counts_dis[10,8,0] = d[85] 
  counts_dis[9,8,0] = d[61] 
  counts_dis[8,8,0] = d[50] 
  counts_dis[7,8,0] = d[71] 
  counts_dis[6,8,0] = d[58] 
  counts_dis[5,8,0] = d[98] 
  counts_dis[4,8,0] = d[93] 
  counts_dis[3,8,0] = d[77] 
  counts_dis[2,8,0] = d[66] 
  counts_dis[1,8,0] = d[76] 
  counts_dis[0,8,0] = d[82] 

  counts_dis[10,8,1] = 85
  counts_dis[9,8,1] = 61 
  counts_dis[8,8,1] = 50 
  counts_dis[7,8,1] = 71
  counts_dis[6,8,1] = 58 
  counts_dis[5,8,1] = 98
  counts_dis[4,8,1] = 93
  counts_dis[3,8,1] = 77 
  counts_dis[2,8,1] = 66 
  counts_dis[1,8,1] = 76 
  counts_dis[0,8,1] = 82 
  ;line2
  counts_dis[10,7,0] = d[51]
  counts_dis[9,7,0] = d[31]
  counts_dis[8,7,0] = d[30]
  counts_dis[7,7,0] = d[53]
  counts_dis[6,7,0] = d[29]
  counts_dis[5,7,0] = d[49]
  counts_dis[4,7,0] = d[87]
  counts_dis[3,7,0] = d[37]
  counts_dis[2,7,0] = d[36]
  counts_dis[1,7,0] = d[34]
  counts_dis[0,7,0] = d[56]

  counts_dis[10,7,1] = 51
  counts_dis[9,7,1] = 31
  counts_dis[8,7,1] = 30
  counts_dis[7,7,1] = 53
  counts_dis[6,7,1] = 29
  counts_dis[5,7,1] = 49
  counts_dis[4,7,1] = 87
  counts_dis[3,7,1] = 37
  counts_dis[2,7,1] = 36
  counts_dis[1,7,1] = 34
  counts_dis[0,7,1] = 56
  ;line3
  counts_dis[10,6,0] = d[28]
  counts_dis[9,6,0] = d[41]
  counts_dis[8,6,0] = d[0]
  counts_dis[7,6,0] = d[1]
  counts_dis[6,6,0] = d[19]
  counts_dis[5,6,0] = d[18]
  counts_dis[4,6,0] = d[45]
  counts_dis[3,6,0] = d[24]
  counts_dis[2,6,0] = d[25]
  counts_dis[1,6,0] = d[44]
  counts_dis[0,6,0] = d[64]

  counts_dis[10,6,1] = 28
  counts_dis[9,6,1] = 41
  counts_dis[8,6,1] = 0
  counts_dis[7,6,1] = 1
  counts_dis[6,6,1] = 19
  counts_dis[5,6,1] = 18
  counts_dis[4,6,1] = 45
  counts_dis[3,6,1] = 24
  counts_dis[2,6,1] = 25
  counts_dis[1,6,1] = 44
  counts_dis[0,6,1] = 64
  ;line4
  counts_dis[10,5,0] = d[81]
  counts_dis[9,5,0] = d[21]
  counts_dis[8,5,0] = d[42]
  counts_dis[7,5,0] = d[43]
  counts_dis[6,5,0] = d[68]
  counts_dis[5,5,0] = d[79]
  counts_dis[4,5,0] = d[67]
  counts_dis[3,5,0] = d[14]
  counts_dis[2,5,0] = d[15]
  counts_dis[1,5,0] = d[89]
  counts_dis[0,5,0] = d[74]

  counts_dis[10,5,1] = 81
  counts_dis[9,5,1] = 21
  counts_dis[8,5,1] = 42
  counts_dis[7,5,1] = 43
  counts_dis[6,5,1] = 68
  counts_dis[5,5,1] = 79
  counts_dis[4,5,1] = 67
  counts_dis[3,5,1] = 14
  counts_dis[2,5,1] = 15
  counts_dis[1,5,1] = 89
  counts_dis[0,5,1] = 74
  ;line5
  counts_dis[10,4,0] = d[96]
  counts_dis[9,4,0] = d[73]
  counts_dis[8,4,0] = d[52]
  counts_dis[7,4,0] = d[91]
  counts_dis[6,4,0] = d[94]
  counts_dis[5,4,0] = d[95]
  counts_dis[4,4,0] = d[92]
  counts_dis[3,4,0] = d[62]
  counts_dis[2,4,0] = d[55]
  counts_dis[1,4,0] = d[90]
  counts_dis[0,4,0] = d[97]

  counts_dis[10,4,1] = 96
  counts_dis[9,4,1] = 73
  counts_dis[8,4,1] = 52
  counts_dis[7,4,1] = 91
  counts_dis[6,4,1] = 94
  counts_dis[5,4,1] = 95
  counts_dis[4,4,1] = 92
  counts_dis[3,4,1] = 62
  counts_dis[2,4,1] = 55
  counts_dis[1,4,1] = 90
  counts_dis[0,4,1] = 97
  ;line6
  counts_dis[10,3,0] = d[88]
  counts_dis[9,3,0] = d[22]
  counts_dis[8,3,0] = d[5]
  counts_dis[7,3,0] = d[54]
  counts_dis[6,3,0] = d[63]
  counts_dis[5,3,0] = d[69]
  counts_dis[4,3,0] = d[48]
  counts_dis[3,3,0] = d[33]
  counts_dis[2,3,0] = d[32]
  counts_dis[1,3,0] = d[72]
  counts_dis[0,3,0] = d[80]

  counts_dis[10,3,1] = 88
  counts_dis[9,3,1] = 22
  counts_dis[8,3,1] = 5
  counts_dis[7,3,1] = 54
  counts_dis[6,3,1] = 63
  counts_dis[5,3,1] = 69
  counts_dis[4,3,1] = 48
  counts_dis[3,3,1] = 33
  counts_dis[2,3,1] = 32
  counts_dis[1,3,1] = 72
  counts_dis[0,3,1] = 80
  ;line7
  counts_dis[10,2,0] = d[75]
  counts_dis[9,2,0] = d[23]
  counts_dis[8,2,0] = d[4]
  counts_dis[7,2,0] = d[6]
  counts_dis[6,2,0] = d[7]
  counts_dis[5,2,0] = d[8]
  counts_dis[4,2,0] = d[9]
  counts_dis[3,2,0] = d[13]
  counts_dis[2,2,0] = d[12]
  counts_dis[1,2,0] = d[20]
  counts_dis[0,2,0] = d[38]

  counts_dis[10,2,1] = 75
  counts_dis[9,2,1] = 23
  counts_dis[8,2,1] = 4
  counts_dis[7,2,1] = 6
  counts_dis[6,2,1] = 7
  counts_dis[5,2,1] = 8
  counts_dis[4,2,1] = 9
  counts_dis[3,2,1] = 13
  counts_dis[2,2,1] = 12
  counts_dis[1,2,1] = 20
  counts_dis[0,2,1] = 38
  ;line8
  counts_dis[10,1,0] = d[65]
  counts_dis[9,1,0] = d[35]
  counts_dis[8,1,0] = d[16]
  counts_dis[7,1,0] = d[17]
  counts_dis[6,1,0] = d[27]
  counts_dis[5,1,0] = d[10]
  counts_dis[4,1,0] = d[11]
  counts_dis[3,1,0] = d[26]
  counts_dis[2,1,0] = d[3]
  counts_dis[1,1,0] = d[2]
  counts_dis[0,1,0] = d[60]

  counts_dis[10,1,1] = 65
  counts_dis[9,1,1] = 35
  counts_dis[8,1,1] = 16
  counts_dis[7,1,1] = 17
  counts_dis[6,1,1] = 27
  counts_dis[5,1,1] = 10
  counts_dis[4,1,1] = 11
  counts_dis[3,1,1] = 26
  counts_dis[2,1,1] = 3
  counts_dis[1,1,1] = 2
  counts_dis[0,1,1] = 60
  ;line9
  counts_dis[10,0,0] = d[83]
  counts_dis[9,0,0] = d[57]
  counts_dis[8,0,0] = d[47]
  counts_dis[7,0,0] = d[46]
  counts_dis[6,0,0] = d[86]
  counts_dis[5,0,0] = d[78]
  counts_dis[4,0,0] = d[59]
  counts_dis[3,0,0] = d[39]
  counts_dis[2,0,0] = d[70]
  counts_dis[1,0,0] = d[40]
  counts_dis[0,0,0] = d[84]
  
  counts_dis[10,0,1] = 83
  counts_dis[9,0,1] = 57
  counts_dis[8,0,1] = 47
  counts_dis[7,0,1] = 46
  counts_dis[6,0,1] = 86
  counts_dis[5,0,1] = 78
  counts_dis[4,0,1] = 59
  counts_dis[3,0,1] = 39
  counts_dis[2,0,1] = 70
  counts_dis[1,0,1] = 40
  counts_dis[0,0,1] = 84
  return,counts_dis
end
