;+
; Mission     : ASO-S/HXI
; Name        : hxi_check_data
; Purpose     : Check if there are lev1 files from 48 hours ago or later in the path
; Category    : imaging
; Calls       : hxi_get_recent_file.pro
; Syntax      :
; Inputs      :
;       trange    - Imaging time period
;       bkg_mode     -  -1:48h before ; 1:48h after
;
;;example     :
;     trange = [['23-feb-23 06:08:27','23-feb-23 06:10:30']]
;     bkg_mode = -1
;     hxi_check_data,trange,bkg_mode
;     
; History     :
;       2023/12/21, Changxue Chen, the first version


pro hxi_check_data,trange,bkg_mode

  delim=path_sep()
  dir_lev1 = hxi_data_paths()
  
  default, dtime, -3600.*48.
  if bkg_mode eq 1 then dtime = 3600.*48.
  this_trange = atime( anytim([trange[0], trange[-1]]) )
  trange_bkg =  atime( anytim( this_trange )  + dtime  ) ;+[-4.1,4.1]
  tarray_input = trange_bkg
  
  if (size(tarray_input))[0] eq 1 then tarray_input =  reform(tarray_input, 2,1)
  this_date = UTC2YYMMDD( anytim(tarray_input[0],/UTC_EXT),/yyyy)
  temp_t= atime(tarray_input[0], /time)
  hr= STRMID(temp_t,strlen(temp_t)-18,2)
  tarray_st = atime(anytim(tarray_input[0]) )
  tarray_et = atime(anytim(tarray_input[-1]) )
  date_st = atime( tarray_st ,/date)
  date_et = atime( tarray_et ,/date)
  hour_st = STRMID(atime(  anytim(tarray_st), /TIME  ),0,2)
  hour_et = STRMID(atime(  anytim(tarray_et), /TIME  ),0,2) 
  nhours = fix((anytim( date_et+' '+hour_et+':00:00.000')-anytim( date_st+' '+hour_st+':00:00.000'))/60./60.)+1 
  all_dates = UTC2YYMMDD( anytim(anytim(tarray_st)+ findgen(nhours)*3600.,/UTC_EXT),/yyyy)
  all_hours = STRMID(atime(  anytim(tarray_st)+ findgen(nhours)*3600., /TIME  ),0,2)
  
  all_files_lev1 = make_array(nhours, value=' ')
  for i=0,nhours-1 do begin
    this_year = STRMID(all_dates[i],0,4)
    this_mon = STRMID(all_dates[i],4,2)
    this_day = STRMID(all_dates[i],6,2)
    this_file_lev1= 'hxi_det_lev10_'+all_dates[i] + '_'+all_hours[i]+'*.fits'
    temp_files = file_search(dir_lev1 ,this_file_lev1) 
    all_files_lev1[i] = hxi_get_recent_file(temp_files[uniq(temp_files)])
  endfor
  
  check_ind = where(file_exist(all_files_lev1) eq 0,check_n)
  if check_n gt 0 then begin
    if bkg_mode eq -1 then result= dialog_message('There is no corresponding data from 48 hours before in the path !', /CENTER,/infor)
    if bkg_mode eq 1 then result= dialog_message('There is no corresponding data from 48 hours after in the path !', /CENTER,/infor)
  endif
  
end