;+
; Mission     : ASO-S/HXI
; Name        : hxi_read_n_fits
; Purpose     : read multi-HXI_ql or HXI_lev1 data files
; Inputs      :
;       filename  - input data files
;       nfiles    - number of input files
;       ftype     - 'ql' or 'sp'
;
; Outputs     :
;       return data structure
;Written: 
; 2021/01/06, Fanxiaoyu Xia
;Modifications:
; 2021/03/29, Fanxiaoyu Xia, modified the way of merging data.
; 2023/03/09, Yang Su, add bkg_edge (data4)

function hxi_read_n_fits, filename, nfiles,ftype =ftype
  
  ;nfiles = n_elements(filename)
  
  case ftype of
    'ql': begin
      
      qlfits_str=hxi_read_fits(filename[0],ftype='ql')
      
;      ; merge structures, yang
;      for i=1,nfiles-1 do begin
;        current_str = hxi_read_fits(filename[i],ftype='ql')
;        qlfits_str = merge_struct( qlfits_str,current_str )
;      endfor
      
      qlfits_times=qlfits_str.qlfits_times
      qlfits_expt = qlfits_str.qlfits_expt ;yang
      qlfits_livet_thinthick =qlfits_str.qlfits_livet_thinthick  ;yang ; 231108
      qlfits_livet_thin = qlfits_str.qlfits_livet_thin ;yang
      qlfits_livet_bg = qlfits_str.qlfits_livet_bg ;yang
      qlfits_bkg_thinthick = qlfits_str.qlfits_bkg_thinthick ;231230,YS
      qlfits_enebins=qlfits_str.qlfits_enebins
      qlfits_data1=qlfits_str.qlfits_data1
      qlfits_data2=qlfits_str.qlfits_data2
      qlfits_data3=qlfits_str.qlfits_data3
      qlfits_data4=qlfits_str.qlfits_data4
      qlfits_data5=qlfits_str.qlfits_data5
      qlfits_data6=qlfits_str.qlfits_data6
      qlfits_data7=qlfits_str.qlfits_data7
      qlfits_data8=qlfits_str.qlfits_data8
      qlfits_data9=qlfits_str.qlfits_data9
      qlfits_data10=qlfits_str.qlfits_data10
      qlfits_data11=qlfits_str.qlfits_data11
      qlfits_flagt =qlfits_str.qlfits_flagt 
      qlfits_flags=qlfits_str.qlfits_flags

      for i=1,nfiles-1 do begin
        qlfits_str= hxi_read_fits(filename[i],ftype='ql')
        qlfits_times=[qlfits_times,qlfits_str.qlfits_times]
        qlfits_expt =[qlfits_expt,qlfits_str.qlfits_expt]  ;yang
        qlfits_livet_thinthick = [qlfits_livet_thinthick, qlfits_str.qlfits_livet_thinthick]  ;yang
        qlfits_livet_thin = [qlfits_livet_thin, qlfits_str.qlfits_livet_thin] ;yang
        qlfits_livet_bg = [qlfits_livet_bg, qlfits_str.qlfits_livet_bg] ;yang
        
        qlfits_bkg_thinthick = [[qlfits_bkg_thinthick],[qlfits_str.qlfits_bkg_thinthick]] ;231230,YS
        
        qlfits_data1=[[qlfits_data1],[qlfits_str.qlfits_data1]]
        qlfits_data2=[[qlfits_data2],[qlfits_str.qlfits_data2]]
        qlfits_data3=[[qlfits_data3],[qlfits_str.qlfits_data3]]
        qlfits_data4=[[qlfits_data4],[qlfits_str.qlfits_data4]]
        qlfits_data5=[qlfits_data5,qlfits_str.qlfits_data5]
        qlfits_data6=[qlfits_data6,qlfits_str.qlfits_data6]
        qlfits_data7=[[[qlfits_data7]],[[qlfits_str.qlfits_data7]]]
        qlfits_data8=[qlfits_data8,qlfits_str.qlfits_data8]
        qlfits_data9=[[qlfits_data9],[qlfits_str.qlfits_data9]]
        qlfits_data10=[qlfits_data10,qlfits_str.qlfits_data10]
        qlfits_data11=[[qlfits_data11],[qlfits_str.qlfits_data11]]

        qlfits_flagt =[qlfits_flagt,qlfits_str.qlfits_flagt]  
        qlfits_flags=[[qlfits_flags],[qlfits_str.qlfits_flags]]
      endfor
 
        qlstr={qlfits_times:qlfits_times, qlfits_expt: qlfits_expt, qlfits_enebins:qlfits_enebins,$
          qlfits_livet_thinthick:qlfits_livet_thinthick, qlfits_livet_thin:qlfits_livet_thin,qlfits_livet_bg:qlfits_livet_bg, $
          qlfits_data1:qlfits_data1, qlfits_data2:qlfits_data2, qlfits_data3:qlfits_data3, $
          qlfits_data4:qlfits_data4, qlfits_data5:qlfits_data5, qlfits_data6:qlfits_data6, $
          qlfits_data7:qlfits_data7, qlfits_data8:qlfits_data8, qlfits_data9:qlfits_data9, $
          qlfits_data10:qlfits_data10,qlfits_data11:qlfits_data11, qlfits_flagt:qlfits_flagt, qlfits_flags:qlfits_flags,$
          qlfits_bkg_thinthick: qlfits_bkg_thinthick}
        return, qlstr
   end

    'sp': begin
      
      spfits_str=hxi_read_fits(filename[0],ftype='sp')
      
;      ; merge structures, yang
;      for i=1,nfiles-1 do begin
;        current_str = hxi_read_fits(filename[i],ftype='sp')
;        spfits_str = merge_struct( spfits_str,current_str )
;      endfor
      
      spfits_times=spfits_str.spfits_times
      spfits_expt= spfits_str.SPFITS_EXPT ;yang
      spfits_livet_spec = spfits_str.spfits_livet_spec  
      
      spfits_spenebins=spfits_str.spfits_spenebins
      spfits_spdata1=spfits_str.spfits_spdata1
      spfits_spdata2=spfits_str.spfits_spdata2
      spfits_lcdata1=spfits_str.spfits_lcdata1
      spfits_lcdata2=spfits_str.spfits_lcdata2
      spfits_lcenebins=spfits_str.spfits_lcenebins
      
      for i=1,nfiles-1 do begin
        spfits_str= hxi_read_fits(filename[i],ftype='sp')
        spfits_times=[spfits_times,spfits_str.spfits_times]
        spfits_expt=[spfits_expt,spfits_str.spfits_expt]  ;yang
        spfits_livet_spec = [[spfits_livet_spec], [spfits_str.spfits_livet_spec]]

        spfits_spdata1=[[[spfits_spdata1]],[[spfits_str.spfits_spdata1]]]
        spfits_spdata2=[[[spfits_spdata2]],[[spfits_str.spfits_spdata2]]]
        spfits_lcdata1=[[[spfits_lcdata1]],[[spfits_str.spfits_lcdata1]]]
        spfits_lcdata2=[[[spfits_lcdata2]],[[spfits_str.spfits_lcdata2]]]
      endfor

      spstr={spfits_times:spfits_times, spfits_expt:spfits_expt, $
        spfits_livet_spec:spfits_livet_spec, spfits_spenebins:spfits_spenebins, $
        spfits_spdata1:spfits_spdata1, spfits_spdata2:spfits_spdata2, $
        spfits_lcdata1:spfits_lcdata1, spfits_lcdata2:spfits_lcdata2, $
        spfits_lcenebins:spfits_lcenebins}
      return, spstr
    end
  endcase

end