;+
; Mission     : ASO-S/HXI
; Purpose:   add some information to the HXI image, Energy spectrum, or other plottings on the plotman windows.
; Category:  auxiliary code for the ASO-S/HXI software
;
; Inputs:
;           plot_obj      - a plotman object
;           t_range       - date-time array, such as [1384168260.0 ,1384170600.0] or ['11-Nov-22 11:12:00.000' ,'11-Nov-22 11:50:24.000']; relate with atime() or anytim()
;           
;(keyword and other input): also see hxi_draw_info_on_plot.pro 
;           spec          - draw information (e.g., t_range) on spectrum plottings, 
;           counts_tmp    - draw counts on the HXI image ,during the imaging process
;           counts_total  - the same as counts_tmp, directly from the input .fits file for maps
;           counts_bg     -                       , directly from the input .fits file for maps
;
;(optional):
;           more_commands - strings of custom command in a array to execute, such as ['oplot,[0,1],[0,1]']
;
;
;Output: drawing
;
;History:
;  2023/04/01, Fu Yu, note: currently, only a few parameters have been used
;       04/09, Yang Su, only show the integer value of the counts
;       04/11, Fu Yu, add obj_pc=pc
;       05/10, Fu Yu, add 'image' to check a plotting bug when total( abs(map.data) ) eq 0.
;       09/01, Changxue Chen, fix a bug, When the input of more_commands keyword is an array
;  2024/02/04, Yang Su, add detid_mm for showing detector range on plot; all inputs for hxi_draw_info_on_plot.pro are now strings.
;    
PRO hxi_draw_info, plot_obj, more_commands = more_commands,$
                   enebins=enebins,method=method,counts_tmp=counts_tmp,$  ;im_info
                   counts_total=counts_total,counts_bg=counts_bg, detid_mm = detid_mm, $   ;240204, YS, add detid_mm
                   t_range=t_range, $
                   spec=spec 

  ;more_commands=['hxi_img_des_on_map,enebins='+'''hello'''+',method='+'''method''']
  ;more_commands=['hxi_img_des_on_map,counts=(*(self.data)).COUNTS']
;  stop
  if plot_obj.get(/plot_type) eq 'image' then begin  ;230510, YF
    tmp_data = plot_obj.get(/data)
    if max(abs(tmp_data.data)) eq 0. then return  ;230601, YF
  endif

  str_tmp=['']
  if keyword_set(counts_tmp) then str_tmp=[str_tmp  + ', counts_total='+'string(round((*(self.data)).COUNTS))']; during the mk-image process

  if keyword_set(counts_total) then str_tmp=[str_tmp  + ', counts_total='+''''+string(round(counts_total))+''''] ;230409, yang su, add fix()
  if keyword_set(counts_bg) then str_tmp=[str_tmp  + ', counts_bg='+''''+string(round(counts_bg))+''''] ;230409, yang su, add fix()
  
  if keyword_set(detid_mm) then str_tmp=[str_tmp  + ', detid_mm = '+'''  D'+string(detid_mm[0],format='(I2)')+' - '+'D'+string(detid_mm[1],format='(I2)')+''''];240204, YS,
  if keyword_set(enebins) then str_tmp=[str_tmp + ', enebins='''+hxi_en2str(enebins)+'''']
  if keyword_set(method) then str_tmp=[str_tmp + ', method='''+method+'''']
  
  if keyword_set(t_range) then begin
    t_range_str = hxi_en2str(t_range,/tim)
    str_tmp=[str_tmp + ', t_range='''+t_range_str+'''']
  endif
  if keyword_set(spec) then str_tmp=[str_tmp + ', /spec']
  
  if str_tmp[0] ne '' then str_tmp=['hxi_draw_info_on_plot,obj_pc=pc '+ str_tmp] ;230411, YF, add obj_pc=pc
  
  if keyword_set(more_commands) then begin
    for i=0, n_elements(more_commands)-1 do str_tmp = str_tmp + ' & ' + more_commands[i] ;0901
  endif
   ; stop
    plot_obj->select
    plot_obj->set, more_commands = str_tmp
    plot_obj->plot
    plot_obj->unselect
END

