;+
; Mission     : ASO-S/HXI
; Puropse     : Save the lightcurve data generated by HXI GUI
; Output      : lcstr -- structure containing lc data
;                       lcstr.sptimes -- DOUBLE[ntime], times
;                       lcstr.enebins -- FLOAT[2, nbins], energy bins
;                       lcstr.data -- FLOAT[nbins, ntime], combined open full detectors
;                       lcstr.databkg -- FLOAT[nbins, ntime], combined background detectors
;                       lcstr.collindex -- INT[8], showing the detectors selection index
;                       lcstr.unit -- a string, indicating the data unit.
; HISTORY:
; Written: Fanxiaoyu Xia, 2022/08/19
; 2022/09/15, Fanxiaoyu Xia, add default filename when save.
; 2023/11/05, Zhentong Li, save the lc of full open and background detectors seperately
;-
pro HXI_save_lc_sav, lcstr

  default, outdir, curdir() + path_sep()   ;230531
  ;dir_hxi=getenv('hxi_db')
  this_ver =  '10'
  tarray=lcstr.sptimes
  exptime = lcstr.spexpt ;231225
  stdate= UTC2YYMMDD( anytim(tarray[0],/UTC_EXT),/yyyy )
  sttime= UTC2YYMMDD( anytim(tarray[0],/UTC_EXT),/yyyy, /HHMMSS)
  ;styear= STRMID(stdate,0,4)
  ;stmon= STRMID(stdate,4,2)

  file_out=outdir+'hxi_lightcurve_'+sttime+'_v'+this_ver+'.sav'
  dir=dialog_pickfile(FILTER=['*.sav*'],TITLE="Select a fits file",$
      file=file_out)
  unit_list=['Counts','Counts/s', 'Counts/s/keV/cm2']
  unit = unit_list[lcstr.evtype]
  if dir ne '' then begin
    data_new=reform((lcstr.data)[*,*,0])
    data_bkg=reform((lcstr.data)[*,*,1])  ;;LZT 20231105
    lcstr={sptimes:lcstr.sptimes, exptime:exptime ,enebins:lcstr.enebins, data:data_new, databkg:data_bkg, collindex:lcstr.collindex, unit:unit}
    save,lcstr,filename=dir,/verb    ;230531
    print,dir+' is saved' 
  endif else begin
    a = dialog_message('No filename is given.')
  endelse

end