;+
;Mission     : ASO-S/HXI
;Purpose     : Main HXI GUI
;Category    : GUI framework
;
;Written: Fanxiaoyu Xia, 2020/10/20
;Modifications:
; 2021/07/21, Fanxiaoyu Xia, added three tabs: 'Quicklook', 'LC/Spectrum' and 'Image' for GUI.
; 2022/03/29, Fanxiaoyu Xia, spectrum_time_range and image_time_ranges can be updated 
;   automaticly with the updated woring_time_range or modified manually.
; 2022/03/30, Fanxiaoyu Xia, the range of imaging detectors D1-D91 can be selected, or the
;   group number of imaging detectors g1-g10 can be selected.
; 2022/03/31, Fanxiaoyu Xia, added checkbox and buttons for flags.
; 2022/07/16 Fanxiaoyu Xia, added a splash screen for hxi GUI. 
; 2022/08/02, Fanxiaoyu Xia, added and realized 'quit' and 'help' button. Removed the detector-
;   checkbox g1-g10 from LC/Spectrum tab.
; 2022/08/04, Fanxiaoyu Xia, made it possible to open and read data from HXI_Q1 or HXI_lev1 fits 
;   files, and updated time range in GUI automaticly. When the filename is wrong or empty, 
;   return an error message.
; 2022/08/07, Fanxiaoyu Xia, Wei Chen, Yang Su, made it possible to check flare ID from working 
;   time range and update time ranges according to the selected flare ID.
; 2022/08/08, Fanxiaoyu Xia, if preparing lightcurves or spectra, two thick detectors, one thin 
;   detector and five backgroound detectors can be selected individually or multi-selected.
; 2022/08/09, Yang su, modified the GUI layout 
; 2022/08/16, Fanxiaoyu Xia, Yang Su, added and realized 'Plot GOES' button.
; 2022/08/19, Fanxiaoyu Xia, added and realized 'Save to sav' and 'Save to fits' buttons.
; 2022/09/16, Fanxiaoyu Xia, restricted spectrum time range to working time range.
; 2022/09/16, Fanxiaoyu Xia, add pointer w_flareid, return multi-Flare_IDs when check Flare_ID, 
;   and one of which can be selected, multi-Flare_IDs are also editable.
; 2022/09/18, Fanxiaoyu Xia, made it possible opening and ploting maps from HXI_imagecube fits.
; 2022/09/19, Fanxiaoyu Xia, modified plotman_widget_resize.pro, so that the draw window can 
;   be zoomed freely with main base. 
; 2023/03/16, Fanxiaoyu Xia, added a droplist to merge time bins for lightcurves.
; 2023/03/17, Fanxiaoyu Xia, added a button to merge spectral data.
; 2023/03/25, Fanxiaoyu Xia, made it possible to select energy bins for lightcurves.
; 2023/03/31, ChangXue Chen, adjusted/added the widgets (Select sub-collimators) to select detectors more convenient, 
;  and add pushbutton with new widget to display counts distribution of each detector. 
; 2023/03/31, Fu Yu, adjust the layout/(Execution order) of GUI to run smoothly. 
;                    Using /nomap is also useful under Windows OS, but not the case in Linux OS.
; ; 2023/05/10, Fu Yu, add 'im_trange_ms' for time intervals by mouse using the method from plotman_obj. 
;         search '@YF 2023 05 10' for the modification (three sites).
; 2023/05/10, Changxue Chen, update the component of displaying the det/bkg counts
; 2023/05/12, Fanxaioyu Xia, added the button 'code' to select 80/64/32 energy bins. 
; 2023/06/13. Changxue Chen, added a judgment: when selecting the imaging-time, if it is not utplot, return
; 2023/09/01, Changxue Chen, fixed a bug: the set time range cannot constrain the imaging time range
; 2023/09/20, Changxue Chen, added new widget to select bkg data for imaging
; 2023/10/17, Changxue Chen, migrated the background selection component to this main program and add some related widgets.
; 2023/10/23, Changxue Chen, adjusted the layout of the background selection component
; 2023/10/24, Zhentong Li, added merge detectors spectra option. 
;         Merging time bins also works for generating spectra files.
; 2023/11/01, Changxue Chen, change keyword input for hxi_get_counts4imaging program
; 2023/12/11, Changxue Chen, change the input for 'Plot Image'
; 2023/12/22, Yang Su, Changxue Chen, add option for 'calibration test'

PRO plotman_widget_resize, event

  widget_control, event.top, get_uvalue=state
  state.plotman_obj -> widget_event, event
  topgeom = widget_info(event.top, /geom)
  maingeom = widget_info (state.widgets.w_maindrawbase, /geom)
    
  yadd = os_family() eq 'Windows' ? 0 : -25
  newx = ( event.x - (topgeom.xsize - maingeom.xsize) ) >300    
  newy = ( event.y - (topgeom.ysize - maingeom.ysize)  ) > 400 
   
  newx_message = newx/12.8  
  
  if newx eq maingeom.xsize and newy eq maingeom.ysize then return
  
  if newx gt 20 and newy gt 20 then begin
    widget_control, state.widgets.w_maindrawbase, xsize=newx, ysize=newy 
    widget_control, state.widgets.w_message,xsize=newx_message 
    current_window_choice = state.plotman_obj -> get(/last_window_choice)
    plotman_window_control_event, 1, saved_uvalue='currmax', state=state 
  endif else begin
    message,/info,'Trying to shrink plotman too much. Not resizing.'
    last_window_choice = state.plotman_obj -> get(/last_window_choice)
    plotman_window_control_event, 1, saved_uvalue=last_window_choice, state=state
  endelse

END

PRO hxi_event,event
  widget_control, event.top, get_uvalue=state
  
  if tag_names(event,/struc) eq 'WIDGET_KILL_REQUEST' then begin
    msg = '    Do you really want to exit the HXI GUI?     '
    answer = xanswer(msg, /str, /suppress, default=1)
    if answer eq 'y' then goto, exit else return
  endif

  if event.id eq event.top then begin
    plotman_widget_resize,event
    return
  endif
  
  widget_control, event.id, get_uvalue=uvalue

  case uvalue of
    'nothing': begin
      numTabs = widget_info(state.wtab, /tab_number)
    end
    
    'ql_trange': begin
      state.ql_trange=atime(event.value)
      ql_range_values=anytim(state.ql_trange)
      state.image_trange=anytim(state.ql_trange)
      state.sp_trange=atime(state.ql_trange)
      state.image_trange[1]=state.image_trange[0]+30.
      *state.im_trange=state.image_trange
      *state.im_trange_values=format_intervals(*state.im_trange,/ut)
      widget_control,state.time_base2,set_value=ql_range_values
      widget_control,state.w_times1,set_value=$
        atime(state.image_trange[0])+' to '+atime(state.image_trange[1])
      widget_control, state.label1, set_value=strmid(state.ql_trange[0],0,18)+ $
        ' to '+strmid(state.ql_trange[1],0,18)
      widget_control, state.label2, set_value=strmid(state.ql_trange[0],0,18)+ $
        ' to '+strmid(state.ql_trange[1],0,18)
      state.qlfits_str=ptr_new(0)
      state.spfits_str=ptr_new(0)
      widget_control, event.top, set_uvalue=state
    end
    
    'data_types': begin
      state.dtype=state.dtype_values[event.index]
      widget_control, event.top, set_uvalue=state
    end
    
    'time_bins': begin
      state.timebins=state.tvalues[event.index]
      widget_control, event.top, set_uvalue=state
    end
    
    'flags': begin
      widget_control, widget_info(event.id, /child ), get_uvalue=state_flag
      state.flag_ptr=ptr_new(state_flag.flags_index)
      ;print,*state.flag_ptr
      widget_control, event.top, set_uvalue=state
    end
      
    'plot': begin
      widget_control, event.id, get_value=value
        case value of 
          'Plot Quicklook': begin
            ql_obj=state.ql_obj 
            qlstr=ql_obj->getdata(time_range=anytim(state.ql_trange), $
              dtype=state.dtype, qlfits_str=*state.qlfits_str)
            state.qlstr=ptr_new(qlstr)
            ql_obj->plotman, qlstr=*state.qlstr, plotman_obj=state.plotman_obj, $
              flag_index=*state.flag_ptr, dtype=state.dtype       
          end
          'Plot LC': begin
            lc_obj=state.lc_obj
            ;print,state.sp_trange
            lc_obj ->plotman,time_range=anytim(state.sp_trange), $
              collindex=*state.coll_ptr, plotman_obj=state.plotman_obj,$
              spfits_str=*state.spfits_str,tbins=state.timebins,evtype=state.evtype,$
              enebins=*state.enebins2
          end
          'Plot Spectra': begin
            spec_obj=state.spec_obj
            ;spstr=spec_obj->getdata(time_range=state.sp_trange, enebins=*state.enebins, collindex=*state.coll_ptr)
            spec_obj ->plotman,time_range=anytim(state.sp_trange), collindex=*state.coll_ptr, $
              enebins=*state.enebins, plotman_obj=state.plotman_obj, combine_check=*state.coll_comb, $
              spfits_str=*state.spfits_str,tbins=state.timebins, evtype=state.evtype
          end

          'Plot Image': begin
            det_range=fix([*state.det_range0,*state.det_range1])
            image_obj=state.im_obj
            det_id = where(*state.coll_ptr2 eq 1); cxchen 0324
            if total(det_id) lt 0 then det_id=0
            ;print,state.im_method
            ;print,state.xc,state.yc,state.nxy,state.psize
            bkgmode = *state.bkg_mode
            bkgtype = *state.bkg_type
            fitmode= *state.bkg_fit

            if exist(*state.bkg_trange) then begin
              bkg_trange = *state.bkg_trange
            endif else begin
              bkg_trange = [anytim(state.ql_trange[0]),anytim(state.ql_trange[0])+30.]
            endelse
 
            image_obj ->plotman,tarray=atime(*state.im_trange),$ 
              enebins=*state.im_enebins, det_id = det_id,$                        
              det_range=det_range,$; collindex=*state.coll_ptr1,$ 
              method=state.im_method ,$      
              xc=state.xc, yc=state.yc,nxy=state.nxy, psize=state.psize, img_calib=state.img_calib, $   ;231222,YS,CC                
              plotman_obj=state.plotman_obj,$
              filename=state.filename,$
              bkg_obj = state.bkg_obj,bkgtype = bkgtype,bkgmode = bkgmode,$ ;cxchen
              fitmode=fitmode,fit_trange=bkg_trange  ;cxchen 0920
          end
          'Plot GOES': begin
            hxi_get_goesdata, trange =state.ql_trange, goes_str = goes_str
            goes_t = atime( anytim(goes_str.goes_t)+ anytim(goes_str.goes_utbase))
            data_obj = obj_new('utplot',goes_t, goes_str.goes_d )
            state.plotman_obj->new_panel,input=data_obj,plot_type='utplot',/ylog 
          end
          
;          'Save to fits': begin
;            print, 'Are you sure to save as fits file?'
;          end
        endcase
      end
      
    'help': begin
        print,'HXI Analysis software instruction ....'
    end
    
    'quit': begin
      goto, exit
    end
    
    'change': begin
      widget_control, state.wtab, SET_TAB_CURRENT=0
    end
    
    'sp_trange': begin   
      state.sp_trange=atime(event.value)
      if anytim(state.sp_trange[0]) lt anytim(state.ql_trange[0]) or $
        anytim(state.sp_trange[1]) gt anytim(state.ql_trange[1]) then begin
          a = dialog_message('Spectrum time range is out of range!')
          state.sp_trange=atime(state.ql_trange)
      endif      
      widget_control,state.time_base2,set_value=anytim(state.sp_trange)
      
      im_trange1=atime(event.value[0])+' to '+atime(event.value[0]+30.)
      tim_tmp = hxi_time_intervals_trans(im_trange1,/str)         ;YF 2023 03 27
      *state.im_trange=tim_tmp.t12
      *state.im_trange_values=tim_tmp.t12_str
    
     ; *state.im_trange_values=im_trange1
      widget_control,state.w_times1,set_value=tim_tmp.t12_str
      state.spfits_str=ptr_new(0)
      widget_control, event.top, set_uvalue=state
    end
    
    'collimator': begin  ;for lc/spec
      widget_control, widget_info(event.id, /child ), get_uvalue=state_coll
      state.coll_ptr=ptr_new(state_coll.coll_index)
      state.coll_comb=ptr_new([state_coll.combine_check, state_coll.combine_bkg_check])  ;1103
      widget_control, event.top, set_uvalue=state
    end
    
    'collimator1': begin  ;for image

      widget_control, widget_info(event.id, /child ), get_uvalue=state_coll
      ;state.coll_ptr1=ptr_new(state_coll.coll_index)
      *state.coll_ptr2 = *state_coll.cdetid
      state.det_range0=ptr_new(state_coll.det_range[0]-1)
      state.det_range1=ptr_new(state_coll.det_range[1]-1)
      widget_control, event.top, set_uvalue=state

    end
 
    'editene': begin
      get_font, font
      list=*state.enevalue
      xtextedit,list,font=font
      q = where(strcompress(list,/remove) ne '', count)
      enebins=unformat_intervals(list[q])
      ;help,enebins
      enevalue=format_intervals(enebins,format='(g10.4)')
      ;help,enevalue
      *state.enebins=enebins
      *state.enevalue=enevalue
      ;relist = 1
      widget_control, state.w_energies,set_value=enevalue
      widget_control, event.top, set_uvalue=state
    end
    
    'editene0': begin
      get_font, font
      list1=*state.enevalue2
      xtextedit,list1,font=font
      q = where(strcompress(list1,/remove) ne '', count)
      enebins2=unformat_intervals(list1[q])
      ;help,enebins2
      enevalue2=format_intervals(enebins2,format='(g10.4)')
      ;help,enevalue
      *state.enebins2=enebins2
      *state.enevalue2=enevalue2
      ;relist = 1
      widget_control, state.w_energies0,set_value=enevalue2
      widget_control, event.top, set_uvalue=state
    end
    
    'save_lc': begin
      lc_obj=state.lc_obj
        lc_obj->output
    end
    
    'save_sp': begin
      spec_obj=state.spec_obj
      collind = *state.coll_ptr
      combine_check=*state.coll_comb
      combine_check[0]= total(collind[0:2]) eq 1 ? 0 : combine_check[0]
      combine_check[1]= total(collind[3:7]) eq 1 ? 0 : combine_check[1]
;      stop
      if total(collind[5:6]) ge 1 then a = dialog_message('Warning: D97 D98 are background detectors located on the edges.')
      if total(collind) eq 0 then begin
        a = dialog_message('Please select detector(s)')
      endif else begin
        dir=dialog_pickfile(/DIRECTORY,/read)
        if dir ne '' then begin 
          spec_obj->output, dir=dir, combine_check=combine_check ;, sepdet=sepdet
        endif else begin
          a = dialog_message('No filename is given')
        endelse
      endelse

    end

    'save_im': begin
      image_obj=state.im_obj
      image_obj->output,tarray=atime(*state.im_trange),$
        enebins=*state.im_enebins
    end
    
    'evtype': begin
      widget_control, state.wtype,get_value=evtype
      state.evtype=evtype
      widget_control, event.top, set_uvalue=state
    end
    ;------------xfxy0512---------------------------
    'code': begin
      infile=hxi_rd_ebins_txt(code=event.index)
      enevalue = rd_ascii(infile, error=error)
      enebins = unformat_intervals(enevalue, ut=ut, err_msg=err_msg)
      *state.enebins=enebins
      *state.enevalue = enevalue
      widget_control, state.w_energies,set_value=enevalue
      widget_control, event.top, set_uvalue=state
    end
    ;----------------------------------------------
 
    'readfile': begin  ;xfxy0510
       infile = dialog_pickfile (path=curdir(), file=file, $
         title = 'Select file to read intervals from')
       if infile eq '' then begin
         a = dialog_message('No file name selected to restore intervals from.  Aborting.')
       endif else begin
         enevalue = rd_ascii(infile, error=error)
         enebins=unformat_intervals(enevalue, ut=ut, err_msg=err_msg)
         *state.enebins=enebins
         *state.enevalue=enevalue
         ;print,*state.enebins
         widget_control, state.w_energies,set_value=enevalue
         widget_control, event.top, set_uvalue=state
       endelse
    end
    
    'readfile0': begin  ;xfxy0510
      infile = dialog_pickfile (path=curdir(), file=file, $
        title = 'Select file to read intervals from')
      if infile eq '' then begin
        a = dialog_message('No file name selected to restore intervals from.  Aborting.')
      endif else begin
        enevalue2 = rd_ascii(infile, error=error)
        enebins2=unformat_intervals(enevalue2, ut=ut, err_msg=err_msg)
        *state.enebins2=enebins2
        *state.enevalue2=enevalue2
        ;print,*state.enebins0
        widget_control, state.w_energies0,set_value=enevalue2
        widget_control, event.top, set_uvalue=state
      endelse
    end
    
    'readfile1': begin  ;xfxy0510
      infile = dialog_pickfile (path=curdir(), file=file, $
        title = 'Select file to read intervals from')
      if infile eq '' then begin
        a = dialog_message('No file name selected to restore intervals from.  Aborting.')
      endif else begin
        enevalue1 = rd_ascii(infile, error=error)
        enebins1=unformat_intervals(enevalue1, ut=ut, err_msg=err_msg)
        *state.im_enebins=enebins1
        *state.im_enevalue=enevalue1
        ;print,*state.enebins
        widget_control, state.w_energies1,set_value=enevalue1
        widget_control, event.top, set_uvalue=state
      endelse
    end
    
    'browse': begin
      fitsname=dialog_pickfile(FILTER=['*.fits*;*.fts*'],TITLE="Select a fits file")
      if(fitsname eq '') then begin
        print,'**you did not choose correct files**'
      endif else begin
        widget_control, event.top, hourglass=1
        print, 'Open file:'+ fitsname
        filename=fitsname
        state.filename=filename
        ftype=hxi_filetype(filename)
        if ftype eq 'im' then begin
          fits2map,filename, maps
          fits_open, filename, fcb, /no_abort, message=err_msg
          fits_close, fcb
          info_ext = where_arr(strlowcase(fcb.extname), strlowcase('PARAMETERS'))
          map_info=mrdfits(filename,info_ext[0],/silent)
          image_obj=state.im_obj
          image_obj ->plotman,data_str=maps,map_info=map_info,$
            plotman_obj=state.plotman_obj
          return
        endif
        if ftype eq '-1' then begin
          a = dialog_message('No HXI file is selected.')
        endif else begin
          data_str= hxi_read_fits(filename,ftype=ftype)
          if ftype eq 'ql' then begin
            *state.qlfits_str=data_str
            fits_times=data_str.qlfits_times
            *state.spfits_str=ptr_new(0)
          endif else begin
          *state.spfits_str=data_str
          fits_times=data_str.spfits_times
          *state.qlfits_str=ptr_new(0)
          endelse
        endelse
        ntimes=n_elements(fits_times)
        ql_range_values=anytim([fits_times[0],fits_times[ntimes-1]])
        widget_control,state.time_base1,set_value=ql_range_values
        state.ql_trange=atime(ql_range_values)
        state.image_trange=anytim(state.ql_trange)
        state.sp_trange=atime(state.ql_trange)
        state.image_trange[1]=state.image_trange[0]+60.
        *state.im_trange=state.image_trange
        *state.im_trange_values=format_intervals(*state.im_trange,/ut)
        widget_control,state.time_base2,set_value=ql_range_values
        widget_control,state.w_times1,set_value=$
          atime(state.image_trange[0])+' to '+atime(state.image_trange[1])
        widget_control, state.label1, set_value=strmid(state.ql_trange[0],0,18)+ $
          ' to '+strmid(state.ql_trange[1],0,18)
        widget_control, state.label2, set_value=strmid(state.ql_trange[0],0,18)+ $
          ' to '+strmid(state.ql_trange[1],0,18)
        widget_control, event.top, set_uvalue=state
      endelse
     end
     
     'flareid': begin
        widget_control, event.id, get_value=value
        case value of
          'Edit': begin
              widget_control,state.w_flareid,get_value=flare_ids
              get_font, font
              list=flare_ids
              xtextedit,list,font=font
              q = where(strcompress(list,/remove_all) ne '', count)
              flare_ids=strtrim(list[q], 1)
              widget_control, state.w_flareid,set_value=flare_ids
              widget_control, event.top, set_uvalue=state
          end
          
          'Check': begin
            flare_ids=hxi_check_flares(state.ql_trange)
            if typename(flare_ids) eq 'STRING' then begin
              widget_control, state.w_flareid,set_value=flare_ids
            endif else begin
              a = dialog_message('No flare is found.')
            endelse
          end
        endcase
      end
      
     'select_id': begin
      widget_control, state.w_flareid,get_value=flare_ids
      flare_id=flare_ids[event.index]
      flist=hxi_rd_flarelist(flare_id)
      if typename(flist) ne 'INT' then begin
        state.ql_trange=atime([anytim(flist.startt)-10.*60.,anytim(flist.endt)+10.*60.])
        ql_range_values=anytim(state.ql_trange)
        state.image_trange=anytim(state.ql_trange)
        state.sp_trange=atime(state.ql_trange)
        state.image_trange[1]=state.image_trange[0]+60.
        *state.im_trange=state.image_trange
        *state.im_trange_values=format_intervals(*state.im_trange,/ut)
        widget_control,state.time_base1,set_value=ql_range_values
        widget_control,state.time_base2,set_value=ql_range_values
        widget_control,state.w_times1,set_value=$
          atime(state.image_trange[0])+' to '+atime(state.image_trange[1])
        widget_control, state.label1, set_value=strmid(state.ql_trange[0],0,18)+ $
          ' to '+strmid(state.ql_trange[1],0,18)
        widget_control, state.label2, set_value=strmid(state.ql_trange[0],0,18)+ $
          ' to '+strmid(state.ql_trange[1],0,18)
        state.qlfits_str=ptr_new(0)
        state.spfits_str=ptr_new(0)
        widget_control, event.top, set_uvalue=state
      endif else begin
        a = dialog_message('No flare is found.')
      endelse
     end
     
     'im_trange': begin
       get_font, font
       list=*state.im_trange_values
       xtextedit,list,font=font
       q = where(strcompress(list,/remove) ne '', count)
;       im_trange=unformat_intervals(list[q],/ut)
;       im_trange_values=format_intervals(im_trange,/ut)
       tim_tmp = hxi_time_intervals_trans(list[q],/str)         ;YF 2023 03 27
       
       list_tmp = ['']   ;cxchen 0901
       for i =0, n_elements(tim_tmp.t12_str)-1 do begin
          if tim_tmp.t12[0,i] lt anytim(state.ql_trange[0]) or $
           tim_tmp.t12[1,i] gt anytim(state.ql_trange[1]) then begin
             a = dialog_message('Imaging time range exceeds set range!')
            
            tim_tmp.t12[0,i]=tim_tmp.t12[0,i]<(anytim(state.ql_trange[1])-30)>anytim(state.ql_trange[0])
            tim_tmp.t12[1,i]=tim_tmp.t12[1,i]<anytim(state.ql_trange[1])>(tim_tmp.t12[0,i]+30)
          endif 
          list_tmp=[list_tmp,atime(tim_tmp.t12[0,i])+' to '+atime(tim_tmp.t12[1,i])]
       endfor
       
       tim_tmp_mod = hxi_time_intervals_trans(list_tmp[1:-1],/str) 
       *state.im_trange=tim_tmp_mod.t12
       *state.im_trange_values=tim_tmp_mod.t12_str
       ;-------------------------------
       widget_control, state.w_times1,set_value=tim_tmp_mod.t12_str 
       widget_control, event.top, set_uvalue=state
     end
     
     ;@YF 2023 05 10 add 'im_trange_ms' for time intervals by mouse
     'im_trange_ms': begin
       plot_type = state.plotman_obj.plot_control.plot_type
       if plot_type ne 'utplot' then begin ; cxchen 20230613 
        BOX_MESSAGE,"The plot type of the current window is not utplot !"
        return 
       endif    
       intervals = atime(*state.im_trange)
       state.plotman_obj->intervals,intervals=intervals
       tim_arr = state.plotman_obj->get(/intervals) ;[2, ntim]
       if n_elements(tim_arr) eq 1 then begin
         print,'No intervals selected!!! Adopt the intervals from the Edit intervals.'
       endif else begin
         tim_arr_str = atime(tim_arr)
         tim_arr_str = strjoin(tim_arr_str,' to ')
         tim_tmp = hxi_time_intervals_trans(tim_arr_str,/str)
         *state.im_trange=tim_tmp.t12
         *state.im_trange_values=tim_tmp.t12_str
         widget_control, state.w_times1,set_value=tim_tmp.t12_str
         widget_control, event.top, set_uvalue=state 
       endelse

     end
     
     'edit_im_ene': begin
       get_font, font
       list=*state.im_enevalue
       xtextedit,list,font=font
       q = where(strcompress(list,/remove) ne '', count)
       im_enebins=unformat_intervals(list[q])
       enevalue=format_intervals(im_enebins,format='(g15.4)')
       *state.im_enebins=im_enebins
       *state.im_enevalue=enevalue
       ;relist = 1
       widget_control, state.w_energies1,set_value=enevalue
       widget_control, event.top, set_uvalue=state
     end
     
     'methods': begin
       state.im_method=state.im_meth[event.index]
       widget_control, event.top, set_uvalue=state
      end
          
     'params': begin
        widget_control, widget_info(event.id, /child ), get_uvalue=state_par
        state.xc=state_par.xc
        state.yc=state_par.yc
        state.nxy=state_par.nxy
        state.psize=state_par.psize
        state.img_calib=*state_par.img_calib   ;231222,YS,CC   
        widget_control, event.top, set_uvalue=state
      end
      
      'det_counts':begin 
        message,/info,'-------Counts Widget Opening-------'
        message,/info,'Please make sure to click the "Plot Data" button first'
        if exist(*state.bkg_str) eq 0 then begin
          result =dialog_message('Please make sure to click the "Plot Data" button first', /CENTER,/infor)
          return
        end
        img_para_change_status = 1 
        tmp_paras_old = *state.bkg_str_old
        tmp_paras_new = *state.bkg_str
        
        time_check =  *state.tmp_im_trange  eq  *state.im_trange
        if total(time_check) lt n_elements(time_check) then begin 
          img_para_change_status=2
        endif else begin
          if exist(tmp_paras_old) and exist(tmp_paras_new) then begin
            for itag=0,n_tags(tmp_paras_new)-1 do begin  ; comparsion for each new input para
              tmp_size_new=size(tmp_paras_new.(itag)) & tmp_size_old=size(tmp_paras_old.(itag))    ;①judge by size
              if n_elements(tmp_size_new) eq n_elements(tmp_size_old) and total((tmp_size_new eq tmp_size_old) eq 0.0) eq 0.0 then begin
                tmp_compare_res = tmp_paras_new.(itag) eq tmp_paras_old.(itag)                     ;②judge by content
                if total(tmp_compare_res eq 0.0) ne 0.0 then img_para_change_status=2.0
              endif else begin
                img_para_change_status=2.0
              endelse
              if img_para_change_status gt 1 then break
            endfor
          endif else begin
            img_para_change_status =2
          endelse
          
        endelse
  
       if img_para_change_status gt 1 then begin
          *state.tmp_im_ebins = *state.im_enebins
          *state.tmp_im_trange = *state.im_trange
          bkg_str = *state.bkg_str
          
          widget_control,state.display_button,get_value = dmode
          if dmode eq 1 then begin
            this_trange =[state.ql_trange[0],state.ql_trange[1]]
            earray = *state.im_enebins
            bkg_obj=state.bkg_obj
            bkg_mode = *state.bkg_mode
            bkg_type = *state.bkg_type
            fit_mode= *state.bkg_fit
            bkg_trange = *state.bkg_trange
            
            bkg_str =bkg_obj.getdata( time_range=this_trange, bkgtype=bkg_type,e_range=earray,bkgmode =bkg_mode,$
              fit_trange = atime(bkg_trange),fitmode=fit_mode)
              
          endif
          
          t_n =(size(*state.tmp_im_trange))[2]
          if (size(*state.tmp_im_trange))[0] eq 1 then t_n=1
;          e_n =(size(*state.tmp_im_ebins))[2]
;          if (size(*state.tmp_im_ebins))[0] eq 1 then e_n=1
          
          counts_str = make_array(t_n,/str)
          counts_bg_str = counts_str
          expts_str = counts_str
          for j=0,t_n-1 do begin
            counts_str[j] = 'counts_'+trim(j)
            counts_bg_str[j] = 'counts_bg_'+trim(j)
            expts_str[j] = 'expts_'+trim(j)
          endfor
          counts_struct = {}
          counts_bg_struct = {}
          expts_struct ={}

          for i = 0, t_n-1 do begin
;            hxi_get_counts4imaging_old,*state.tmp_im_ebins,(*state.tmp_im_trange)[*,i], counts, counts_bg $
;              ,tot_counts, tot_counts_bg, cent_xy, tarray_out,expt
            
            hxi_get_counts4imaging, *state.tmp_im_ebins, (*state.tmp_im_trange)[*,i],  counts_str=counts4imaging_struct

            counts = counts4imaging_struct.counts_subcoll
            tarray_out = counts4imaging_struct.tarray_out
            expt = counts4imaging_struct.expt

            expts_struct = create_struct(expts_struct,[expts_str[i]],expt)
            counts_struct = create_struct(counts_struct,[counts_str[i]],counts)
            
            this_ind = where(anytim(bkg_str.tarray) ge anytim(tarray_out[0]) and anytim(bkg_str.tarray) le anytim(tarray_out[-1]))
            counts_bg_struct = create_struct(counts_bg_struct,[counts_bg_str[i]],bkg_str.bkg_counts[*,this_ind,*])
        
          endfor
          *state.expts = expts_struct
          *state.det_counts = counts_struct
          *state.det_bg_counts = counts_bg_struct 
        endif
        countwidget,*state.tmp_im_trange,*state.tmp_im_ebins,*state.det_counts,*state.det_bg_counts $
          , *state.expts,group_leader=event.top
      end
      
      'BKG_TIME':begin
        
        if exist(*state.bkg_trange) then begin
          intervals = *state.bkg_trange
        endif else begin
          intervals = [anytim(state.ql_trange[0]),anytim(state.ql_trange[0])+30.]
        endelse
        state.plotman_obj->intervals,intervals=intervals
        tim_arr = state.plotman_obj->get(/intervals) ;[2, ntim]
        if n_elements(tim_arr) lt 2 then begin
          tim_arr = intervals
        endif
        tim_arr_str = atime(tim_arr)
        tim_arr_str = strjoin(tim_arr_str,' to ')
        tim_tmp = hxi_time_intervals_trans(tim_arr_str,/str)
        *state.bkg_trange=tim_tmp.t12 ; anytim-sec
       ; widget_control, state.bkg_times,set_value=tim_tmp.t12_str
        widget_control, event.top, set_uvalue=state
      end
      
      
      'BKG_TYPE':begin
        widget_control,state.btype_button,get_value = btype
        case btype of 
          0:bkg_type = 'bkg_det' 
          1:bkg_type = 'fit_curve'
          2:bkg_type = '48h_modu_det'
        endcase
        *state.bkg_type = bkg_type
;        if *state.bkg_type eq '48h_modu_det' then result =dialog_message($
;          'If using BKG-3,  please ensure that there is corresponding data before or after 48 hours in the path', /CENTER,/infor)

      end
      
      'BKG_MODE':begin
        widget_control,state.bmode_button,get_value = bmode
        case bmode of
          0:bkg_mode = -1
          1:bkg_mode = 1
         ; 2:bkg_type = 2
        endcase
        *state.bkg_mode = bkg_mode
        hxi_check_data,atime(*state.im_trange), bkg_mode
      end      
      
      'FIT_METHOD':begin
        bfit = event.index
        case bfit of
          0:bkg_fit = '0Poly'
          1:bkg_fit = '1Poly'
          2:bkg_fit = 'BKG Profile'
        endcase
        *state.bkg_fit = bkg_fit
      end

      'PLOT_BKG':begin
        ;this_trange = atime(*state.im_trange)
        this_trange =[state.ql_trange[0],state.ql_trange[1]]
        earray = *state.im_enebins
        widget_control,state.display_button,get_value = dmode
        if dmode eq 1 then earray=[[10.,20.],[20.,50.],[50.,100.],[100.,300.]]
        bkg_mode = *state.bkg_mode
        bkg_obj=state.bkg_obj
        bkg_type = *state.bkg_type
        fit_mode= *state.bkg_fit 
        *state.bkg_str_old = *state.bkg_str
        
        if exist(*state.bkg_trange) then begin
          bkg_trange = *state.bkg_trange
        endif else begin
          bkg_trange = [anytim(state.ql_trange[0]),anytim(state.ql_trange[0])+30.]
          *state.bkg_trange = bkg_trange
        endelse
       
        bkg_str =bkg_obj.getdata( time_range=this_trange, bkgtype=bkg_type,e_range=earray,bkgmode =bkg_mode , $
          fit_trange = atime(bkg_trange),fitmode=fit_mode)

        bkg_obj->plotman, bkg_str = bkg_str, plotman_obj=state.plotman_obj, time_range =this_trange,$
          bkgtype=bkg_type,e_range=earray
        
        *state.bkg_str = bkg_str
        end
      
      'HELP_BKG':begin
        bkg_help_widget,group_leader = event.top
        end

  endcase
  
  RETURN
  
  exit:
  widget_control, event.top, /destroy

END


PRO hxi, time_interval=time_interval
 
  HXI_ENV_SET  
  ;setpath=hxi_data_paths()
 
  device, get_screen_size=screen_size
  xoffset=screen_size[0]/10
  yoffset=screen_size[1]/8
  xsize = screen_size[0]*0.36 > 680.
  ysize = screen_size[1]*0.6 > 750.
  
  if not valid_time_range (time_interval) then begin
    ;  ql_trange = anytim(['2023/02/23 06:08:00','2023/02/23 06:20:00']) ;yang; 230519
      ;ql_trange = anytim(['2022/11/11 11:39:30','2022/11/11 11:46:40']) ;yang
;      ql_trange = anytim(['2023/05/01 13:05:00','2023/05/01 13:12:00']) ;yang
    ;  ql_trange = anytim(['2023/02/17 19:10:30','2023/02/17 20:20:00'])
    ql_trange = anytim(['2023/05/09 03:42:00','2023/05/09 04:06:00'])
  endif
  
  if not obj_valid(ql_obj) then ql_obj = hxi_quicklook()
  if not obj_valid(spec_obj) then spec_obj = hxi_spectrum()
  if not obj_valid(lc_obj) then lc_obj= hxi_lightcurve()
  if not obj_valid(im_obj) then im_obj = hxi_image()
  if not obj_valid(bkg_obj) then bkg_obj =hxi_bkgdata()
  
  tlb=widget_base(title='HXI Main Window',column=2, /base_align_top, $   
    mbar=mbar,xoffset=xoffset,  $
    /tlb_size_events,/tlb_kill)  

  w_file=widget_button(mbar,/menu,value='File',uvalue='file')
  open=widget_button(w_file,value='Open',uvalue='browse')
  state={mbar:mbar,w_file:w_file}
  widget_control,tlb,set_uvalue=state
 
;-------------------------------------------------
system_tag = os_family()
;if system_tag eq 'Windows' then begin  ;20230331, move to the bottom of this .pro for other system 
;    plotman_obj=obj_new('plotman',mainbase=tlb,/multi_panel,widgets=widgets, /nomap)
;  widget_control,widgets.w_maindrawbase,xsize=xsize,ysize=ysize 
;  geom = widget_info(widgets.w_maindrawbase, /geom)
;  w_splashdraw=widget_draw(widgets.w_maindrawbase,xsize=geom.scr_xsize,$
;    ysize=geom.scr_ysize)
;  widget_control,widgets.w_message,xsize=geom.scr_xsize/6.25
;  widget_control,widgets.w_message,xsize=geom.scr_xsize/12.8
;  helpmenu=widget_button(mbar,value='Help')
;
;  wbase = widget_base(tlb, column=2, frame=1)
;endif else begin 
;  wbase = widget_base(tlb, /column, frame=0) 
;  wbase = widget_base(tlb, /column, /base_align_top)  
;endelse
;
wbase = widget_base(tlb, /column, frame=0)
wbase = widget_base(tlb, /column, /base_align_top)
;-------------------------------------------

  wtab = widget_tab(wbase, location=location, uvalue='nothing')
 
  wt1 = widget_base(wtab, title='Quicklook', /column)
  time_base1=hxi_cw_ut_range(wt1, value=ql_trange, $  
    label= 'Working Time Range: ', uvalue='ql_trange')                      

  flare_base=widget_base(wt1, /column, /frame, space=50)
  base0=widget_base(flare_base, /column)
  base00=widget_base(base0, /row, space=50,/BASE_ALIGN_CENTER) ;YS, 230513;230601 
  label0 = widget_label(base00, value='Flare ID: ')
  w_flareid = widget_droplist(base00, xsize=150, value ='', $   ;YS, 230513, 150-200
    uvalue='select_id' )
  butt0=widget_button(base00, value='Check',$  
    /frame, uvalue='flareid')
  butt0=widget_button(base00, value='Edit', $  
    /frame, uvalue='flareid')
  

  dtype_base=widget_base(wt1, /column, /frame)
  base1=widget_base(dtype_base, /column)
  base10 = widget_base (base1, /row, space=120,/ALIGN_CENTER ,/BASE_ALIGN_CENTER) ;YS, 230513;230601 
  wlabel = widget_label(base10, value='Select Data to Plot: ')
  dtype_values =['Thin','ThinThick','BKG_ThinThick','BKG_edge','BKG_cent', 'livet','Tot_image', $
    'Tot_spec', 'Pointing', 'Roll_Angle', 'Temper', 'hv', 'xyz']   ;add livet,bkg_cent, bkg_edge, 230309, yang su;231108,YS, thick->thinthick; 231230, added BKG_ThinThick
  w_datatype = widget_droplist(base10, value =dtype_values, $
    uvalue='data_types')
  widget_control, w_datatype, set_droplist_select=0
  
  fdefault=replicate(0,7)
  wflag_base = hxi_flag_widget(wt1, uvalue='flags')
  
  plot_base = widget_base (wt1, /column, /frame)
  base3=widget_base(plot_base, /column)
  base30= widget_base(base3, /row, /align_center, space=50) ;YS, 230513
  tmp = widget_button(base30, value='Plot Quicklook', uvalue='plot' ,/frame)
  tmp = widget_button(base30, value='Plot GOES', uvalue='plot' ,/frame)
  tmp = widget_button(base30, value='Help', uvalue='help' ,/frame)
  tmp = widget_button(base30, value='Quit', uvalue='quit' ,/frame)
  ; b2=widget_button(wt1, value=, uvalue='set', /frame)

  wt2 = widget_base(wtab, title='LC/Spectrum', /column)
  label_base = widget_base(wt2, /column, /frame)
  base4= widget_base(label_base, /column)
  ;base40= widget_base(base4, /row, space=66)
  base40= widget_base(base4, /row, space=230)
  label0 = widget_label(base40, value='Working Time Range:')
  butt0=widget_button(base40, value='change' ,/frame, uvalue='change')
  base41= widget_base(base4, /column)
  ql_trange=atime(ql_trange)
  label1 = widget_label(base41, value=strmid(ql_trange[0],0,18)+$
    '  to  '+strmid(ql_trange[1],0,18))   ;230519, YS, '  to  '
  time_base2=hxi_cw_ut_range2(wt2, value=ql_trange, $  
    label='LC/Spectrum Time Range: ', uvalue='sp_trange',/noreset)
  
  text1=['10.0 to  20.0',  '20.0 to  50.0', '50.0 to 100.0', '100.0 to 300.0']
  enebins1=unformat_intervals(text1)
  enevalue1=text1
  ene_base = widget_base(wt2,/column ,/frame)
  base5=widget_base(ene_base,/column)
  base50 = widget_base(base5, /row, /BASE_ALIGN_CENTER) ;230601,YS
  w_energies0 = widget_droplist(base50, title='LC Energy Bins (keV):     ', $
    value=enevalue1, uvalue='nothing')
  ;base61 = widget_base(base6, /row, space=20)
  base51 = widget_base(base5, /row, space=65)
  tmp = widget_button(base51, value='Edit Intervals', uvalue='editene0')
  tmp = widget_button(base51, value='Read Intervals from File', uvalue='readfile0')  
  
  cdefault=[0,0,1,0,0,0,0,0]
  coll_comb0 = [0,0]   ;1103
  wcoll_base = hxi_coll_widget( wt2, uvalue='collimator',cdefault=cdefault,coll_comb=coll_comb0)

  units_base=widget_base(wt2,/column ,/frame)
  base7=widget_base(units_base,/column)
  base70=widget_base(base7,/row)  ;230601
  label3 = widget_label(base70, value='Units: ',/align_left)
  type_list=['Counts','Count Rate', 'Count Flux']
  wtype=cw_bgroup(base70, type_list,/row,/exclusive, uvalue='evtype',space=0)
  WIDGET_CONTROL, wtype, set_value=1 ;230601
  
  plot_base3 = widget_base(wt2, /column, /frame)
  ;base10=widget_base(plot_base3, /column)
  base10=widget_base(plot_base3, /column, space=100);yang
  base100= widget_base(base10, /row, /align_center,/BASE_ALIGN_CENTER, space=10)  ;xfxy0510
  wlabel2 = widget_label(base100, value='Select Tbins to Plot: ')
  tvalues =['0.25','0.5','1.0', '2.0','4.0']
  w_datatype2 = widget_droplist(base100, value =tvalues, $
    uvalue='time_bins' ) 
    WIDGET_CONTROL, w_datatype2, SET_DROPLIST_SELECT=2  
  tmp = widget_button(base100, value='Plot LC', uvalue='plot' ,/frame)
  ;tmp = widget_button(base100, value='Show Flags', uvalue='show' ,/frame)
  tmp = widget_button(base100, value='Save to SAV', uvalue='save_lc' ,/frame)   ;xfxy0510; YS 230513

  empty_base = widget_base(wt2,/column )
  basee=widget_base(empty_base,/column)
  label7 = widget_label(basee, value=' ',/align_left)

   ;-----------xfxy0512--------------------------
  infile=hxi_rd_ebins_txt(code=1)
  text = rd_ascii(infile, error=error)
  enebins0=unformat_intervals(text)
  enevalue0=text
  ene_base = widget_base(wt2,/column ,/frame)
  base6=widget_base(ene_base,/column)
  base60 = widget_base(base6, /row, space=18)  
  tmp = widget_label(base60, value='Spectrum Energy Bins (keV):  ', /align_left)  

  base61 = widget_base(base6, /row, space=60 )
  w_code = widget_droplist(base61, value=['binning code 0: 80 bins','binning code 1: 64 bins','binning code 2: 32 bins'], uvalue='code') ;230601, YS
  WIDGET_CONTROL, w_code, SET_DROPLIST_SELECT=1
  w_energies = widget_droplist(base61, value=enevalue0, uvalue='nothing')
  base62 = widget_base(base6, /row, space=70) ;yang
  tmp = widget_button(base62, value='Edit Energy Intervals', uvalue='editene')
  tmp = widget_button(base62, value='Read Intervals from File', uvalue='readfile')  
  ;------------------------------------------------

;  ;select time_bins
;  tbins_base = widget_base(wt2,/column ,/frame)
;  base5=widget_base(tbins_base,/column)
;  base50=widget_base(base5,/column)
;  w_tbins = widget_droplist(base50, title='Time Bins (s): ', $
;    value=['2','4','6','8','10','12'])   

  plot_base2 = widget_base(wt2, /column, /frame)
  base9=widget_base(plot_base2, /column)
  base90= widget_base(base9, /row, /align_center, space=50) ; 230513, YS
  base91= widget_base(base9, /row, /align_center, space=95) 
  tmp = widget_button(base90, value='Plot Spectra', uvalue='plot' ,/frame)
  ;tmp = widget_button(base90, value='Plot Merged Spectrum', uvalue='plot' ,/frame)
  tmp = widget_button(base90, value='Save to Fits', uvalue='save_sp' ,/frame)
  tmp = widget_button(base90, value='Help', uvalue='help' ,/frame)
  tmp = widget_button(base90, value='Quit', uvalue='quit' ,/frame)

  wt3=widget_base(wtab,title='Image', /column)
  label_base1 = widget_base(wt3, /column, /frame)
  base10= widget_base(label_base1, /column)
  base100= widget_base(base10, /row, space=230)
  labe20 = widget_label(base100, value='Working Time Range:')
  butt1=widget_button(base100, value='change' ,/frame, uvalue='change')
  base101= widget_base(base10, /column)
  ql_trange=atime(ql_trange)
  label2 = widget_label(base101, value=strmid(ql_trange[0],0,18)+$
    ' to '+strmid(ql_trange[1],0,18))
    
  time_base3 = widget_base(wt3,/column ,/frame)
  base13=widget_base(time_base3,/column)
  base130 = widget_base(base13, /row, space=20); 180 -> 100,  @YF 2023 05 10; YS 230512 100->20
  labe30 = widget_label(base130, value='Image Time Range:')
  base131 = widget_base(base13, /row)

  image_trange = [anytim(ql_trange[0]),anytim(ql_trange[0])+30.]
  im_trange0=atime(anytim(ql_trange[0]))+'  to  '+atime(anytim(ql_trange[0])+30.)
  w_times1 = widget_droplist(base131,value=im_trange0,$
     uvalue='nothing')
  tmp = widget_button(base130, value='Edit Intervals', uvalue='im_trange')
  tmp = widget_button(base130, value='Select with mouse', uvalue='im_trange_ms'); @YF 2023 05 10; YS 230512 change item name

  ene_base1 = widget_base(wt3,/column ,/frame)
  base11=widget_base(ene_base1,/column)
  base110 = widget_base(base11, /row, space=125)
  im_enevalue0='20.00-30.00'
  im_enebins0=unformat_intervals(im_enevalue0)
  labe40 = widget_label(base110, value='Image Energy Bins (keV): ')
  w_energies1 = widget_droplist(base110, value=im_enevalue0, uvalue='nothing')
  base111 = widget_base(base11, /row, space=55)
  tmp = widget_button(base111, value='Edit Energy Bins', uvalue='edit_im_ene')
  tmp = widget_button(base111, value='Read Energy Bins from File', uvalue='readfile1')  
  ;--------------------------------------------
  wcoll_base2 = widget_base(wt3,/frame,/colum)
  cdefault2 = replicate(0,91)
  base15 = hxi_coll_widget3(wcoll_base2, uvalue='collimator1')
 
  bkg_str = {}  ;ccx 20231017 
  tmp_base= widget_base(wcoll_base2)
  tmp = widget_label(tmp_base,value ='Select Background Data (Please Refer to Help):',xoffset = 10)
  
  base_bkg = widget_base(wcoll_base2,/row)
  base_btype = widget_base(base_bkg,/colum,/frame)
  type_list = ['1: Background Monitors Data','2: Fitting of Non-flare Data','3: Other Orbit Counts']
  btype_button = cw_bgroup(base_btype,type_list,/colum,/exclusive,uvalue='BKG_TYPE',space=10)
  bkg_type = 'bkg_det' ; button=0
  WIDGET_CONTROL, btype_button, set_value=0
  
  base_bmode = widget_base(base_bkg,/colum,space = 5,/frame,/align_cent)

  base_bmode1 = widget_base(base_bmode,/row)
  tmp = widget_label(base_bmode1,value ='Default (not recommended unless..)')
  
  base_bmode2 = widget_base(base_bmode,/row)
  tmp = widget_button(base_bmode2,value=' Select BKG Time ',uvalue = 'BKG_TIME');
  bkg_trange = []

  method_list = ['0Poly','1Poly','BKG Profile'];,'This E profile']
  ;method_list = ['0Poly','1Poly','2Poly'];,'This E profile']
  fit_button = widget_droplist(base_bmode2,value = method_list,uvalue='FIT_METHOD')
  bkg_fit = '0Poly'
;  
  
  base_bmode3 = widget_base(base_bmode,/row)
  mode_list = ['48h Before','48h Ahead']; 'Average 48h']
  bmode_button = cw_bgroup(base_bmode3,mode_list,/row,/exclusive,uvalue='BKG_MODE',space=0)
  bkg_mode = -1 ;before 48h
  WIDGET_CONTROL, bmode_button, set_value=0
  ;----------------------------
  display_base=widget_base(wcoll_base2,/row,/ALIGN_left,space=0,/frame) ;20231204
  tmp = widget_label(display_base,value = 'Display: ')
  display_list = ['Selected energy bins','Default energy bins']
  display_button = cw_bgroup(display_base,display_list,/row,/exclusive,uvalue='nothing',space=5)
  WIDGET_CONTROL, display_button, set_value=0
  ;----------------------------------------------
  base_disp = widget_base(wcoll_base2,/row,/ALIGN_left,space=20) ;20231204
 
  ;base_check = widget_base(base_disp,/colum,/align_cent,space=20);,space=15
  tmp = widget_button(base_disp,value='  Plot Data  ', uvalue='PLOT_BKG')
  tmp = widget_button(base_disp, value='Counts of Different Detectors', uvalue='det_counts', $
    /ALIGN_CENTER);,xsize=tmp_xsize;,ysize=36) ;YS, 230512
  quit = widget_button(base_disp,value='  Help  ',uvalue='HELP_BKG')
  
  ;tmp_xsize = 0.2*screen_size[0]
 
  ;--------------------------------------------------
  method_base1 = widget_base(wt3,/column ,/frame)
  base13=widget_base(method_base1,/column)
  base130 = widget_base(base13, /row, space=215) 
  im_meth=['HXI_Clean','HXI_BackProj','HXI_FFitting','Vis_Clean','Vis_BP','VIS_FF','MEM_GE']
  labe50 = widget_label(base130, value='Imaging Method: ') ;YS, 230512
  w_methods = widget_droplist(base130, value=im_meth, uvalue='methods')
  
  wparams_base1 = hxi_image_params( wt3, uvalue='params')  
 
  plot_base3 = widget_base (wt3, /column, /frame)
  base12=widget_base(plot_base3, /column)
  base120= widget_base (base12, /row, /align_center, space=50) ;yang; 230513, YS
  tmp = widget_button(base120, value='Plot Image', uvalue='plot' ,/frame)
  tmp = widget_button(base120, value='Save to fits', uvalue='save_im' ,/frame)
  tmp = widget_button(base120, value='Help', uvalue='help' ,/frame)
  tmp = widget_button(base120, value='Quit', uvalue='quit' ,/frame)
  
  ;if system_tag ne 'Windows' then begin ; 2023 04 01
   ;@YF20230331, move to here for smoothly starting the software------------------------------------
    plotman_obj=obj_new('plotman',mainbase=tlb,/multi_panel,widgets=widgets)
    widget_control,widgets.w_maindrawbase,xsize=xsize,ysize=ysize
    geom = widget_info(widgets.w_maindrawbase, /geom)
    w_splashdraw=widget_draw(widgets.w_maindrawbase,xsize=geom.scr_xsize,$
      ysize=geom.scr_ysize)
    ;widget_control,widgets.w_message,xsize=geom.scr_xsize/6.25
    widget_control,widgets.w_message,xsize=geom.scr_xsize/12.8 
    helpmenu=widget_button(mbar,value='Help')
  ;------------------------------------------------------------------------------------------------
  ;endif
  
  ;
  state = { tlb:tlb, wtab:wtab, dtype:'Thin', dtype_values:dtype_values, $
    label1:label1, label2:label2, ql_trange:ql_trange, $
    plotman_obj:plotman_obj,w_splashdraw:w_splashdraw,spec_obj:spec_obj, lc_obj:lc_obj, $
    ql_obj:ql_obj, im_obj:im_obj,$
    time_base2:time_base2, w_times1:w_times1, base15:base15,$
    sp_trange:ql_trange,  qlstr:ptr_new(0), image_trange:image_trange,$
    widgets:widgets, enevalue:ptr_new(enevalue0), enevalue2:ptr_new(enevalue1), $
    im_enevalue:ptr_new(im_enevalue0),$
    flag_ptr:ptr_new(fdefault), im_enebins:ptr_new(im_enebins0),$
    tmp_im_ebins:ptr_new(tmp_im_ebins0),tmp_im_trange:ptr_new(tmp_im_trange0),$  
    im_trange:ptr_new(image_trange),im_trange_values:ptr_new(im_trange0),$
    enebins:ptr_new(enebins0), enebins2:ptr_new(enebins1),$
    w_energies:w_energies, w_energies0:w_energies0, w_energies1:w_energies1,$
    coll_ptr:ptr_new(cdefault),coll_ptr2:ptr_new(cdefault2),$
    coll_comb: ptr_new(coll_comb0), $ 
    det_range0:ptr_new('18'),det_range1:ptr_new('90'),$  ;CXC, 240228
    det_counts:ptr_new(det_counts),det_bg_counts:ptr_new(det_bg_counts),expts:ptr_new(expts),$ 
    im_method:'HXI_Clean', im_meth:im_meth,$
    xc:409.,yc:280.,nxy:101,psize:2., img_calib:0, qlfits_str:ptr_new(0),$    ;231222,YS,CC, img_calib   
    filename:'',spstr:ptr_new(0),time_base1:time_base1,spfits_str:ptr_new(0),$
    wtype:wtype, evtype:1, w_flareid:w_flareid, timebins:1.,tvalues:tvalues, $  ;230601, YS, evtype 1,timebins:1.  
    bkg_type:ptr_new(bkg_type),bkg_mode:ptr_new(bkg_mode),bkg_obj:bkg_obj,bkg_str:ptr_new(bkg_str),$
    bkg_trange:ptr_new(bkg_trange),bkg_fit:ptr_new(bkg_fit),bkg_str_old:ptr_new(bkg_str),$
    fit_button:fit_button,btype_button:btype_button,bmode_button:bmode_button, $ ;20230901,0917,1017 ccx 
    display_button:display_button} ; 1204 ccx
  widget_control,tlb, set_uvalue=state
  widget_control,tlb, /realize, update=0
  hxi_fpage
  widget_control,tlb, /map, /update, /show

  xmanager, 'hxi', tlb, event='hxi_event', /no_block
END
