;+
; Mission     : ASO-S/HXI
; Name        : hxi_bkgdata
; PURPOSE     : Define hxi_bkgdata Object 
; CONTRUCTION:
;  o = Obj_New('hxi_bkgdata') 
; HISTORY:
;  Writen by Changxue Chen, 2023/09/20
;  2023/10/24, Changxue Chen, ploted 'time rebined' background data
;  2023/11/01, Changxue Chen, added a new fitting method - 'BKG Profile', delete '2Poly'; 
;       change the Keyword input for counts4imaging program
;  2023/11/06, Changxue Chen, modified the input structure (tmp_str_bkg) when merging data to a 4-second resolution    
;  2023/11/09, Changxue Chen, added bkg_counts4plot and counts4plot two variables to the bkg_str for plotting time series of (bkg)counts

function hxi_bkgdata::getdata, time_range=time_range, bkgtype=bkgtype,e_range=e_range,bkgmode=bkgmode $
  ,fitmode=fitmode,fit_trange=fit_trange;, bkg_str=bkg_str
  
  tarray = [atime(anytim(time_range[0,0])),atime(anytim(time_range[1,-1]))]
  earray = e_range & tarray = tarray
;  hxi_get_counts4imaging_old,earray, tarray, counts_this, counts_bg_this, tot_counts, tot_counts_bg, cent_xy $
;    ,tarray_out, expt_this

  hxi_get_counts4imaging, earray, tarray,  counts_str=counts_str,cent_xy=cent_xy, prod=prod, filename=filename

  counts_this = counts_str.counts_subcoll
  counts_bkgdet_center = reform(counts_str.counts_bkgdet[0,*,*])
  counts_bkgdet_edge =counts_str.counts_bkgdet_edge
  tot_counts= counts_str.counts_totdet
;  tot_counts_bg= counts_str.counts_totdet_bkg
  tarray_out = counts_str.tarray_out
  expt_this = counts_str.expt
  livet = counts_str.livet_subcoll
  livet_bg = counts_str.livet_bkgdet

  ;tarray_out - UT format
  sz = size(counts_this) ;[ndet,tims,neng]
  ndet = sz[1] & ntims =sz[2] & neng =sz[3]
  if sz[0] eq 2 then neng = 1
  
  counts_rate = make_array(ndet,ntims,neng)
  counts_rate_bkg = make_array(ndet,ntims,neng)
  counts_bg_this =  make_array(ndet,ntims,neng)
  
  for i=0,neng-1 do begin
    for j=0,ndet-1 do begin
     counts_bg_this[j,*,i] = (counts_bkgdet_center[*,i]+counts_bkgdet_edge[*,i])/2.
    ; counts_bg_this[j,*,i] = counts_bkgdet_edge[*,i]
      counts_rate[j,*,i] = counts_this[j,*,i]/expt_this
      counts_rate_bkg[j,*,i] = counts_bg_this[j,*,i]/expt_this
    endfor
  endfor
  crate = counts_rate
  ;---------------
  if not keyword_set(bkgtype) then bkgtype = 'bkg_det'
  
    case bkgtype of
      'bkg_det': begin
        bkg_crate = counts_rate_bkg
        bkg_counts = counts_bg_this
        bkg_counts4plot= reform(counts_bg_this[0,*,*])
        bkgmode = ' '
        fitmode = ' '
      end
      'fit_curve': begin
       if not keyword_set(fitmode) then fitmode ='0Poly'
       
       if fitmode eq 'BKG Profile' then begin
         ;-----------------------------------------------
         expts = expt_this
         bkg_counts_avg = average(counts_bg_this,1)
         times=tarray_out
         sz = size(counts_this) ;[ndet,tims,neng]
         neng =sz[3]& nt = sz[2]
         if sz[0] eq 2 then neng = 1
        
         info={ttime:0d, counts:0d, ltime: 0d, exptime:0d }
         tmp_str_bkg = replicate(info,nt)
         tmp_str_bkg.ttime  = anytim(times)
         tmp_str_bkg.counts = bkg_counts_avg[*,0]
         tmp_str_bkg.ltime  = expts
         tmp_str_bkg.exptime  = expts
         
         ;tmp_str_bkg = {ttime:anytim(times),exptime:expts,counts:bkg_counts_avg[*,0],ltime:expts}
         result_bkg =  HXI_REBIN_EXPTIM(tmp_str_bkg,timebin=4)
         tmp_times = atime(result_bkg.ttime)
         ntimes = n_elements(tmp_times)
         bkg_crate_avg_rb = make_array(ntimes,neng)
         bkg_crate_avg =  make_array(nt,neng)
         
         times_origin = total(expts,/cum)-expts/2.
         times_rebin = total(result_bkg.ltime,/cum)-result_bkg.ltime/2.
         
         for ii =0,neng-1 do begin
           tmp_str_bkg.counts = bkg_counts_avg[*,ii] ;1204 ccx
           ;tmp_str_bkg = {ttime:anytim(times),exptime:expts,counts:bkg_counts_avg[*,ii],ltime:expts}
           result_bkg =  HXI_REBIN_EXPTIM(tmp_str_bkg,timebin=4)
           bkg_crate_avg_rb[*,ii] = result_bkg.counts /result_bkg.ltime
           ;bkg_crate_avg[*,ii] = interpol(smooth(bkg_crate_avg_rb[*,ii],10,/edge_mirr),times_rebin,times_origin)
           bkg_crate_avg[*,ii] = interpol(bkg_crate_avg_rb[*,ii],times_rebin,times_origin)
         endfor         
         ;--------------------
       endif
       
        hxi_bkg_fit,fit_trange =fit_trange,erange =earray,fitmode=fitmode ,expts = expt_this $
          , fit_crate=fit_crate,fit_counts=fit_counts $ 
          ,im_trange = time_range , bkg_counts_rate = bkg_crate_avg, $
          bkg_counts4plot = bkg_counts4plot
          ;stop
        bkg_crate = fit_crate
        bkg_counts = fit_counts
        bkgmode=' '
      end
      '48h_modu_det': begin
        if not keyword_set(bkgmode) then bkgmode = -1
        case bkgmode of
          -1:dtime=-3600*48.
          1:dtime=3600*48.
          2: ; add avg 48h data 
        endcase
        this_trange = [tarray_out[0],tarray_out[-1]]
        counts_48h = make_array(ndet,ntims,neng)
        bkg_counts4plot = make_array(ntims,neng)
 
        for i=0,neng-1 do begin
          earr = earray[*,i]
          hxi_get_bkgdata_48h,this_trange, expt_this, earr,crate_bkg,crate_bkgdet,counts_bkg,counts_bkgdet,totcounts_totdet_48h, dtime = dtime
          counts_rate_bkg[*,*,i]= crate_bkg ; *  ( total( counts_bg_this[0,*,i],2) / total( counts_bkgdet[0,*],2) )[0]
          counts_48h[*,*,i] = counts_bkg
          bkg_counts4plot[*,i] = totcounts_totdet_48h
        endfor
        bkg_crate = counts_rate_bkg
        bkg_counts = counts_48h
        fitmode = ' '
      end
      else:
    endcase
    
    bkg_str = {tarray:tarray_out,earray:earray,counts:counts_this,bkg_counts:bkg_counts,crate:crate, $
      bkg_crate:bkg_crate,expts:expt_this,bkgtype:bkgtype,bkgmode:bkgmode,fitmode:fitmode, $
      bkg_counts4plot:bkg_counts4plot,counts4plot:tot_counts}

  return,bkg_str
end

pro hxi_bkgdata::plotman,plotman_obj=plotman_obj, time_range=time_range,bkgtype=bkgtype,e_range=e_range $
  ,bkgmode=bkgmode ,fitmode=fitmode,fit_trange=fit_trange, bkg_str=bkg_str

  if not keyword_set(bkg_str) then begin
    bkg_str=self->getdata(time_range=time_range, bkgtype=bkgtype,e_range=e_range,bkgmode=bkgmode $
        ,fitmode=fitmode,fit_trange=fit_trange)
  endif
  if not is_struct(bkg_str) then begin
    a = dialog_message ('No plotman data found.')
    return
  endif
  expts = bkg_str.expts
;  ind = where(expts lt 0.99,nind)
  bkg_counts_avg = bkg_str.bkg_counts4plot ;& bkg_counts_avg = average(bkg_counts,1)
  counts_avg = bkg_str.counts4plot ;& counts_avg = average(counts,1)
  times=bkg_str.tarray
  enebins=bkg_str.earray
;  crate=bkg_str.crate & crate_avg = average(crate,1)
;  bkg_crate=bkg_str.bkg_crate & bkg_crate_avg = average(bkg_crate,1)
  nsize=n_elements(size(counts))
  sz = size(counts_avg) ;[ndet,tims,neng]
  nt = sz[1]  & neng =sz[2]
  if sz[0] eq 1 then neng = 1
  
  ;rebin --------------------------------------------
  info={ttime:0d, counts:0d, ltime: 0d, exptime:0d}
  tmp_str_bkg = replicate(info,nt)
  tmp_str_bkg.ttime  = anytim(times)
  tmp_str_bkg.counts = bkg_counts_avg[*,0]
  tmp_str_bkg.ltime  = expts
  tmp_str_bkg.exptime  = expts
 ; tmp_str_bkg = {ttime:anytim(times),exptime:expts,counts:bkg_counts_avg[*,0],ltime:expts}
  result_bkg =  HXI_REBIN_EXPTIM(tmp_str_bkg,timebin=4)
  tmp_times = atime(result_bkg.ttime)
  ntimes = n_elements(tmp_times)
  bkg_crate_avg_rb = make_array(ntimes,neng)
  crate_avg_rb = make_array(ntimes,neng)

  for ii =0,neng-1 do begin
    tmp_str_bkg.counts = bkg_counts_avg[*,ii]
 ;  tmp_str_bkg = {ttime:anytim(times),exptime:expts,counts:bkg_counts_avg[*,ii],ltime:expts}
  ;  tmp_str_count ={ttime:anytim(times),exptime:expts,counts:counts_avg[*,ii],ltime:expts}
    tmp_str_count =tmp_str_bkg & tmp_str_count.counts = counts_avg[*,ii]
    result_bkg =  HXI_REBIN_EXPTIM(tmp_str_bkg,timebin=4)
    result_count = HXI_REBIN_EXPTIM(tmp_str_count,timebin=4)
    bkg_crate_avg_rb[*,ii] = result_bkg.counts /result_bkg.ltime
    crate_avg_rb[*,ii] = result_count.counts/result_count.ltime
  endfor
; -------------------------------------------------------------
 ; ymax=max(crate)
;stop
  des_tmp='ASO-S HXI | bkgdata quicklook'
  
  comb_crate = make_array(ntimes,neng*2)
  comb_crate[*,0:neng-1] = crate_avg_rb
  comb_crate[*,neng:2*neng-1] = bkg_crate_avg_rb
    data_obj = obj_new('utplot',tmp_times, comb_crate, /ylog)
    
      dim_id=make_array(2*neng,/string)
      for i=0,neng-1 do dim_id[i]=trim(enebins[0,i])+' to '+trim(enebins[1,i])+' keV (Data with BK)'
      for i=neng,2*neng-1 do dim_id[i]=trim(enebins[0,i-neng])+' to '+trim(enebins[1,i-neng])+' keV (BK)'
      
      data_obj -> set, dim1_id = dim_id
      plotman_obj ->new_panel,input=data_obj, plot_type='utplot',/ylog,xst=1,yst=1, $
        ytitle=['Counts s!u-1!n detector!u-1!n'],description= des_tmp+'-> '+bkg_str.bkgtype+' mode',$
        title = des_tmp+': '+bkg_str.bkgtype+' mode'
end


pro hxi_bkgdata::set, time_range=time_range,  bkgtype=bkgtype,e_range=e_range,bkgmode=bkgmode,$
  dtype=dtype

  if keyword_set(time_range) then begin
    *self.time_range=ptr_new(time_range)
  endif

  if keyword_set(e_range) then begin
    *self.e_range=ptr_new(e_range)
  endif
end

pro hxi_bkgdata__define
  self = {hxi_bkgdata, bkg_str:ptr_new(),e_range:ptr_new(),bkgtype:'bkg_det',time_range:ptr_new()}
end