;+
; Mission     : ASO-S/HXI
; PURPOSE:
;  Define hxi_quicklook Object
; CONTRUCTION:
;  o = Obj_New('hxi_quicklook') or  o = hxi_quicklook()
; HISTORY:
;  Writen by Fanxiaoyu Xia, 2021/11/27
; MODIFICATIONS:
; 2021/04/27, Fanxiaoyu Xia, added keyword flag_index for procedure hxi_quicklook::plotman, and 
;   get informations of flags by hxi_quicklook::getdata.
; 2022/07/16, Fanxiaoyu Xia, if filename is empty or wrong, return an error message.
; 2022/08/08, Fanxiaoyu Xia, added keyword qlfits_str for procedure hxi_quicklook::getdata.
; 2022/08/18, Changxue Chen, Fu Yu， modified display mode of flags.
; 2023/04   , Fu Yu, using hxi_draw_info for more_commands.
; 2023/04   , Changxue Chen, modify the flag time array based on Lev-q1 data
; 2023/06/30. Changxue Chen, fixed a bug where flags don't display in some situations
; 2023/07/17, Yang Su, adjusted plot configration/settings for different types of QL data (dtype)
; 2023/12/30, Yang Su, add 'bkg_thinthick'

function hxi_quicklook::getdata, time_range=time_range, dtype=dtype, qlfits_str=qlfits_str
  if is_struct(qlfits_str) eq 0 then begin
    stime=atime(time_range[0])
    etime=atime(time_range[1])
    hxi_find_data,stime,etime,fileout=filename,/levql
 
    nfiles=n_elements(filename)
    nofiles=0
    for i=0,nfiles-1 do begin
      if filename[i] eq '' then begin
        nofiles=1
      endif
    endfor
    if nofiles eq 1 then return,-1

    if nfiles eq 1 then begin
      qlfits_str= hxi_read_fits(filename,ftype='ql')
    endif else begin
      qlfits_str= hxi_read_n_fits(filename,nfiles,ftype='ql')
    endelse
  endif
  
  qlfits_times=qlfits_str.qlfits_times
  qlfits_expt = qlfits_str.qlfits_expt  
  tindex=where(qlfits_times ge time_range[0] and qlfits_times le time_range[1])
  times=qlfits_times[tindex]
  
  qlfits_flagt = qlfits_str.qlfits_flagt  ; 0407
  flag_tindex=where(qlfits_flagt ge time_range[0] and $
    qlfits_flagt le time_range[1])
  flag_times=qlfits_flagt[flag_tindex]
  qlfits_flags=qlfits_str.qlfits_flags
  flags=qlfits_flags[*,flag_tindex]

  expt = qlfits_expt[tindex]
  enebins=qlfits_str.qlfits_enebins
  data1=qlfits_str.qlfits_data1[*,tindex]
  data2=qlfits_str.qlfits_data2[*,tindex]
  data3=qlfits_str.qlfits_data3[*,tindex]
  data4=qlfits_str.qlfits_data4[*,tindex]
  data5=qlfits_str.qlfits_data5[tindex]
  data6=qlfits_str.qlfits_data6[tindex]
  data7=qlfits_str.qlfits_data7[*,*,tindex]  ;0824
  data8=qlfits_str.qlfits_data8[tindex]
  data9=qlfits_str.qlfits_data9[*,tindex]
  data10=qlfits_str.qlfits_data10[tindex]
  data11=qlfits_str.qlfits_data11[*,tindex]
  data12=qlfits_str.qlfits_livet_thin[tindex]
  data13=qlfits_str.qlfits_bkg_thinthick[*,tindex] ;231231,

  if keyword_set(dtype) then begin
    case dtype of 
      'ThinThick': begin  ;231108,YS
        qlstr={times:times, enebins:enebins, data:data1, flags:flags,flag_times:flag_times}
      end
      'BKG_ThinThick': begin  ;231230,YS
        qlstr={times:times, enebins:enebins, data:data13, flags:flags,flag_times:flag_times}
      end
      'Thin': begin
        qlstr={times:times, enebins:enebins, data:data2, flags:flags,flag_times:flag_times}
      end
      'BKG_cent': begin
        qlstr={times:times, enebins:enebins, data:data3, flags:flags,flag_times:flag_times}
      end
      'BKG_edge': begin
        qlstr={times:times, enebins:enebins, data:data4, flags:flags,flag_times:flag_times}
      end
      'Tot_image': begin
        qlstr={times:times, enebins:enebins, data:data5, flags:flags,flag_times:flag_times}
      end
      'Tot_spec': begin
        qlstr={times:times, enebins:enebins, data:data6, flags:flags,flag_times:flag_times}
      end
      'Pointing': begin
        qlstr={times:times, enebins:enebins, data:data7, flags:flags,flag_times:flag_times}
      end
      'Roll_Angle': begin
        qlstr={times:times, enebins:enebins, data:data8, flags:flags,flag_times:flag_times}
      end
      'Temper': begin
        qlstr={times:times, enebins:enebins, data:data9, flags:flags,flag_times:flag_times}
      end
      'hv': begin
        qlstr={times:times, enebins:enebins, data:data10, flags:flags,flag_times:flag_times}
      end
      'xyz': begin
        qlstr={times:times, enebins:enebins, data:data11, flags:flags,flag_times:flag_times}
      end
      'livet': begin
        qlstr={times:times, enebins:enebins, data: f_div(data12,expt), flags:flags,flag_times:flag_times}
      end
    endcase
  endif else begin
    qlstr={times:times, enebins:enebins, ThinThick:data1, BKG_ThinThick:data13, Thin:data2, $
      Backg_cent:data3, Backg_edge:data4,Tot_image:data5, Point:data7, Roll_Angle:data8, $
      Temper:data9, hv:data10, xyz:data11,livet:data12}  ;231231,YS,BKG_Thinthick
  endelse
  return,qlstr
end

pro hxi_quicklook::plotman,plotman_obj=plotman_obj, time_range=time_range, $
  dtype=dtype, qlstr=qlstr, flag_index=flag_index
  if not keyword_set(qlstr) then begin
    qlstr=self->getdata(time_range=time_range, dtype=dtype)
  endif
  if not is_struct(qlstr) then begin
    a = dialog_message ('No plotman data found.')
    return
  endif
  times=qlstr.times
  enebins=qlstr.enebins
  data=qlstr.data
  nsize=n_elements(size(data))
  if nsize eq 5 then begin
    data=transpose(data)
  endif 
  dsize=size(data)
  ymax=max(data)
  
  des_tmp='ASO-S HXI | quicklook ' 
  if dtype eq 'Pointing' then begin
      data_obj = obj_new('xyplot', reform(data[0,0,*])-1023.5,2048.5-reform(data[1,0,*])-1023.5 )  ;240124,YS
      plotman_obj ->new_panel,input=data_obj, plot_type='xyplot',  $
        ytitle='SAS pixel',xtitle='SAS pixel', title='HXI/SAS info: Location of solar disk center',xst=1,yst=1,psym=0,$
        xrange=[min(data[0,0,*])-5.,max(data[0,0,*])+5]-1023.5,$
        yrange=[min(2048.5-data[1,0,*])-5.,max(2048.5-data[1,0,*])+5]-1023.5 ,$
        description= des_tmp+ '-> Pointing'  ;231218,YS
  end else begin
    data_obj = obj_new('utplot',times, data, /ylog)

    if dsize[0] eq 2 and dsize[2] eq 4 then begin
      dim_id=make_array(4,/string)
      for i=0,3 do dim_id[i]='HXI '+strmid(enebins[i,0],6,3)+' to '+strmid(enebins[i,1],6,3)+' keV'
      data_obj -> set, dim1_id = dim_id
    endif else if dtype eq 'Tot_image' or dtype eq 'Tot_spec' then begin
      dim_id=strmid(enebins[0,0],5,3)+' to '+strmid(enebins[-1,1],5,4) +' keV'
      data_obj -> set, dim1_id =dim_id
    endif
    
    ;['Thin','ThinThick','BKG_edge','BKG_cent', 'livet','Tot_image', $
    ;'Tot_spec', 'Pointing', 'Roll_Angle', 'Temper', 'hv', 'xyz']
    ;231217,YS
    if dtype eq 'livet' or dtype eq 'Roll_Angle' or dtype eq 'Temper' or dtype eq 'hv' or dtype eq 'xyz' then begin
      case dtype of
        'livet' : plotman_obj ->new_panel,input=data_obj, plot_type='utplot',ylog=0, $
          ytitle=['live time (percent)'],description= des_tmp+'-> Live time',$
          title = des_tmp+': Live time'
        
        'Roll_Angle': plotman_obj ->new_panel,input=data_obj, plot_type='utplot',ylog=0, $
          ytitle=['degree'],description= des_tmp+'-> roll_angle',$
          title = des_tmp+': roll_angle'
          
        'Temper': plotman_obj ->new_panel,input=data_obj, plot_type='utplot',ylog=0, $
          ytitle=['T (degree)'],description= des_tmp+'-> temper',$
          title = des_tmp+': temper.'
          
        'hv': plotman_obj ->new_panel,input=data_obj, plot_type='utplot',ylog=0, $
          ytitle=['V'],description= des_tmp+'-> hv', yrange=[(min(data)-20)>0,max(data)+20],$
          title = des_tmp+': hv'
          
        'xyz': plotman_obj ->new_panel,input=data_obj, plot_type='utplot',ylog=0, $
          ytitle=[' '],description= des_tmp+'-> xyz',$
          title = des_tmp+': xyz'
 
      endcase
 
    endif else begin
      plotman_obj ->new_panel,input=data_obj, plot_type='utplot',/ylog, $
        ytitle=['Counts s!u-1!n detector!u-1!n'],description= des_tmp+'-> '+dtype,$
        title = des_tmp+': '+dtype 
    endelse
  endelse
 
  more_commands=['']
  flags=qlstr.flags 
  flag_times = qlstr.flag_times
  ind=where(flag_index ne 0)
  ;stop
  if ind[0] eq -1 then begin
    print,'No flag is selected.'
  endif else begin
    
    for j=0,n_elements(ind)-1 do begin
      der_flags = reform(flags[ind[j],1:-1]-flags[ind[j],0:-2])
      ind_st= where(der_flags eq 1)+1
      ind_ed = where(der_flags eq -1)
      if flags[ind[j],0]  eq 1 then ind_st = [0,ind_st] & ind_st=ind_st[uniq(ind_st)]
      if flags[ind[j],-1] eq 1 then ind_ed = [ind_ed, n_elements(der_flags)-1] & ind_ed=ind_ed[uniq(ind_ed)]

      d_ind = ind_ed-ind_st
      dt_limit = 5 & ind_ind = where(d_ind gt dt_limit)
      if ind_ind[0] ge 0 then begin
        ind_st=ind_st[ind_ind] & ind_ed = ind_ed[ind_ind]
        sbar = atime(flag_times[ind_st]) & ebar =atime(flag_times[ind_ed])
        flag_ind = ind[j]
;        sbar_ = '' & ebar_ = sbar_
;        for k=0,n_elements(sbar)-1 do sbar_ = sbar_+''''+sbar[k]+''''+','
;        for k=0,n_elements(ebar)-1 do ebar_ = ebar_+''''+ebar[k]+''''+','
;        sbar_ = strmid(sbar_,0,strlen(sbar_)-1) & ebar_ = strmid(ebar_,0,strlen(ebar_)-1)
        sbar_ = "['"+strjoin(sbar,"','")+"']"
        ebar_ = "['"+strjoin(ebar,"','")+"']"
        more_commands = [more_commands,'hxi_draw_flags,sbar='+sbar_+',ebar='$
          +ebar_+',flag_ind ='+trim(flag_ind)+',utplot =1,n_bar =n_bar']
      endif
    endfor
   ; stop
    if n_elements(more_commands) ge 2 and dtype ne 'Pointing' then begin  ;231218,YS
     hxi_draw_info, plotman_obj, more_commands = more_commands[1:-1] ;2023 04 simplify
    endif
  endelse
end


pro hxi_quicklook::set, time_range=time_range, dtype=dtype

  if keyword_set(time_range) then begin
    *self.time_range=time_range
  endif

  if keyword_set(collnum) then begin
    self.collnum=ptr_new(collnum)
  endif

  if keyword_set(energy_bins) then begin
    self.energy_bins=ptr_new(energy_bins)
  endif
end

pro hxi_quicklook__define
  self = {hxi_quicklook, qlstr:ptr_new(), tstart:0.d0, tend:0.d0,  collnum:ptr_new(),$
    energy_bins:ptr_new(), data_units:0}
end
