;+
; Mission     : ASO-S/HXI
; PURPOSE:
;  Define hxi_spectrum Object
; CONTRUCTION:
;  o = Obj_New('hxi_spectrum') or  o = hxi_spectrum()
; HISTORY:
;  Writen by Fanxiaoyu Xia, 2021/11/27
;  2022/07/16, Fanxiaoyu Xia, added an error message when filename is empty or wrong.
;  2022/08/08, Fanxiaoyu Xia, added keyword spfits_str for function hxi_spectrum::getdata.
;  2022/08/12, Fanxiaoyu Xia, added lables for detectors when plotting spectra.
;  2022/08/19, Fanxiaoyu Xia, added procedure hxi_spectrum::setdata.
;  2022/03/21, Fanxiaoyu Xia, added keyword merge.
;  2023/04/01, Fu Yu, add title, description, and other params (using hxi_draw_info.pro) for the plot
;  2023/10/20, Zhentong Li, added keyword tbins for hxi_spectrum::getdata
;  2023/11/04, Zhentong Li, added plot combined spectra function
;  2023/11/07, Zhentong Li, fixed the plotted lc units

pro hxi_spectrum::set, time_range=time_range, time_interval=time_interval, energy_bins=energy_bins, $
  collnum=collnum, data_units=data_units, _extra=_extra  
  
  if keyword_set(time_range) then begin
    self.tstart=anytim(time_range[0])
    self.tend=anytim(time_range[1])
  endif

  if keyword_set(collnum) then begin
    self.collnum=ptr_new(collnum)
  endif

  if keyword_set(energy_bins) then begin
    self.energy_bins=ptr_new(energy_bins)
  endif
end

function hxi_spectrum::getdata, time_range=time_range, collindex=collindex, $
  spfits_str=spfits_str, enebins=enebins, evtype=evtype, tbins=tbins, combine_check=combine_check
  
  if is_struct(spfits_str) eq 0 then begin
    stime=atime(time_range[0])
    etime=atime(time_range[1])
    hxi_find_data,stime,etime,fileout=filename,/lev10
    nfiles=n_elements(filename)
    nofiles=0
    for i=0,nfiles-1 do begin
      if filename[i] eq '' then begin
        nofiles=1
      endif
    endfor
    if nofiles eq 1 then return,-1
    
    if nfiles eq 1 then begin
      spfits_str= hxi_read_fits(filename,ftype='sp')
    endif else begin
      spfits_str= hxi_read_n_fits(filename,nfiles,ftype='sp')
    endelse
  endif
  
  spfits_expt=spfits_str.spfits_expt  
  spfits_livet = spfits_str.spfits_livet_spec  
  spfits_times=spfits_str.spfits_times
  tindex=where(spfits_times ge anytim(time_range[0]) and  spfits_times le anytim(time_range[1])) 
  sptimes=spfits_times[tindex]
  spfits_expt=spfits_expt[tindex]

  splivet4fits = spfits_livet[*,tindex]
  sptimes4fits = transpose( [ [spfits_times[tindex]], [spfits_times[tindex]+spfits_expt[tindex]] ] )
  nt_intveral = n_elements(tindex)

;  data_modu=spfits_str.spfits_spdata1[*,*, tindex]
;  data1=total(data_modu[*,*,*],3)
  data_spec=spfits_str.spfits_spdata2[*,*, tindex]
  data2=total(data_spec[*,*,*],3) 
  odata=make_array(80, 8, /float)
  for i=0, 7 do begin
    odata[*, i]=data2[*, i]
  endfor
;  odata=data2
  
  spfits_spenebins=spfits_str.spfits_spenebins
  spfits_spenebins=transpose(spfits_spenebins)
  n_bins=n_elements(enebins[0,*])
  data=make_array(n_bins,8,/float)
  spec4fits = fltarr(n_bins, 8, nt_intveral) 
  for i=0, n_bins-1 do begin
    ind0=where(enebins[0, i] le spfits_spenebins[0,*])
    ind1=where(enebins[1, i] le spfits_spenebins[1,*])
    ind0=ind0[0]
    ind1=ind1[0]
    data[i,*]=total(odata[ind0:ind1,*],1)
    spec4fits[i,*,*] = total(data_spec[ind0:ind1,*,*],1)
    the_dur = total(splivet4fits,2) 
  endfor

 ;;---- time rebin, LZT 20231020 --- 
 if keyword_set(tbins) and tbins ne 0.25 then begin
   ;datastr={ttime:sptimes, exptime:spfits_expt, counts:data, ltime:splivet4fits}

   szc= size(spec4fits,/dim)
   info={ttime:0d, exptime:0. ,$
     counts: fltarr(szc[0],szc[1]), $
     ltime: fltarr(szc[1]) }
   datastr = replicate(info,szc[2])
   datastr.ttime = sptimes
   datastr.exptime =spfits_expt
   datastr.counts = spec4fits
   datastr.ltime = splivet4fits


   outspec=HXI_REBIN_EXPTIM2(datastr,timebin=tbins)
   spec4fits=outspec.counts
   sptimes4fits=outspec.ttime   ;sptimes=outspec.ttime
   sptimes4fits=transpose( [[sptimes4fits], [sptimes4fits+outspec.exptime]] )
   splivet4fits=outspec.ltime
 endif
 
  if keyword_set(collindex) then begin
    spind=where(collindex[0:7] eq 1) 
    if spind[0] eq -1 then begin  
;      a = dialog_message('No collimator is selected.')
      data=-1  
      spec4fits = -1       
    endif else begin
      data=data[*, spind] 
      tspec4fits=spec4fits
      spec4fits[*,*,*]=0.
      spec4fits[*,spind,*] = tspec4fits[*,spind,*]
      splivet4fits[spind, *] = splivet4fits[spind, *]  
      the_dur = the_dur[spind] 
    endelse
  endif
  
;  area = hxi_constant(/detector_area)   ;LZT, 20231104
;  if keyword_set(evtype) then begin
;    ndet=(size(data))[0]
;    if ndet eq 1 then ewid = get_edges(enebins,/width) else ewid = rebin(get_edges(enebins,/width), n_elements(get_edges(enebins,/width)), (size(data))[2],/sample)  
;    case evtype of 
;      0: print, '0'
;      1: data = data/transpose( rebin( the_dur, (size(the_dur))[1], (size(data))[1] ))
;      2: data = data/transpose( rebin( the_dur, (size(the_dur))[1], (size(data))[1] ))/ewid/area ;;(!pi*(2.2/2.)^2.)
;    endcase 
;  endif

spstr={sptimes:sptimes, sptimes4fits:sptimes4fits, enebins:enebins, data:data, spec4fits:spec4fits, $
  collindex:collindex, splivet4fits:splivet4fits}
  return,spstr
end

pro hxi_spectrum::plotman,plotman_obj=plotman_obj, time_range=time_range, spfits_str=spfits_str, $
  collindex=collindex, enebins=enebins, evtype=evtype, merge=merge, tbins=tbins, combine_check=combine_check

  spstr = self->getdata(time_range=time_range, collindex=collindex, enebins=enebins, evtype=evtype, tbins=tbins) 
  spind = where(collindex[0:7] eq 1)
  open_ind = where(collindex[0:2] eq 1, n_open)
  bkg_ind = where(collindex[3:7] eq 1, n_bkg)
  if spind[0] eq -1 then spstr = -1
  if not is_struct(spstr) then begin
    a = dialog_message('No plotman data found. Please select detector(s) first')
    return
  endif
  if n_elements(size(spstr.data)) lt 4 then begin
    if spstr.data eq -1 or spstr.spec4fits eq -1 then begin 
      return
    endif
  endif
;stop
  dim_title=['t1 (thick)','t2 (thick)','t3 (thin)','b1','b2','b3','b4','b5']
  dim_title0=['t1','t2','t3','b1','b2','b3','b4','b5']
    
  e_bins=(enebins[0,*]+enebins[1,*])/2.
  e_bins=reform(e_bins)
  data_obj = obj_new('xyplot',e_bins, spstr.data)
  
  dim1=combine_check[0] eq 0 ? n_open : (1<n_open)
  dim2=combine_check[1] eq 0 ? n_bkg : (1<n_bkg)
  
  adata = fltarr(n_elements(e_bins), dim1+dim2)
  dim_id = strarr(dim1+dim2)
  
  IF n_open ne 0 then begin
    IF combine_check[0] eq 0 or n_open eq 1 THEN BEGIN
      for i = 0, n_open-1 do dim_id[i]=dim_title[open_ind[i]]
;      for i = 0, n_open-1 do adata[*,i]=(spstr.data)[*,i]
      for i = 0, n_open-1 do begin
        zeroarr = intarr(8)
        zeroarr[open_ind[i]]=1
        sep_spec = hxi_combine_det_spec(spstr, ebins=ebins,detectors=zeroarr,evtype=evtype)
        adata[*,i]=sep_spec.data
      endfor
      
    endif else begin
      dim_id[0] = strjoin(dim_title0[open_ind],' + ')
      comb_spec = hxi_combine_det_spec(spstr, ebins=ebins,detectors=collindex,evtype=evtype)
      adata[*,0] = comb_spec.data
    ENDELSE
  ENDIF

  IF n_bkg ne 0 then begin
    IF combine_check[1] eq 0 or n_bkg eq 1 THEN BEGIN
      for i = 0, n_bkg-1 do dim_id[i+dim1]=dim_title[bkg_ind[i]+3]
;      for i = 0, n_bkg-1 do adata[*,i+dim1]=(spstr.data)[*,i+n_open]
      for i = 0, n_bkg-1 do begin
        zeroarr = intarr(8)
        zeroarr[bkg_ind[i]+3]=1
        sep_spec = hxi_combine_det_spec(spstr, ebins=ebins,detectors=zeroarr,evtype=evtype,/bkg)
        adata[*,i+dim1]=sep_spec.data
      endfor
      
    endif else begin
      dim_id[dim1] = strjoin(dim_title0[bkg_ind+3],' + ')
      comb_spec = hxi_combine_det_spec(spstr, ebins=ebins,detectors=collindex,evtype=evtype,/bkg)
      adata[*,dim1] = comb_spec.data
    ENDELSE
  ENDIF

  data_obj = obj_new('xyplot',e_bins, adata)
  data_obj -> set, dim1_id = dim_id
    
  title_list = ['Counts vs Energy', 'Count Rate vs Energy', 'Count Flux vs Energy']                       
  des_tmp = hxi_en2str(atime(time_range),/tim) + ' | ASO-S HXI | Energy Spectrum -> '+ title_list[evtype]
  if not keyword_set(plotman_obj) then begin
    plotman_obj=obj_new('plotman')
    plotman_obj->new_panel,input=data_obj,plot_type='xyplot',/xlog,/ylog, description='ASO-S HXI | Spectrum ' 
    return
  endif
    
  type_list=['Counts','Counts s!u-1!n', 'Counts s!u-1!n keV!u-1!n cm!u-2!n']
  plotman_obj ->new_panel,input=data_obj, plot_type='xyplot', /xlog,/ylog, ytitle=type_list[evtype], xtitle='Energy (keV)', $
    title= 'ASO-S/HXI Energy Spectrum:'+ blank(2)+ title_list[evtype], description = des_tmp         
  self->setdata,data_str=spstr
;  t_range_str = hxi_en2str(atime(time_range),/tim)
;  more_commands=['hxi_img_des_on_map,/spec, t_range='''+t_range_str+'''']
;  hxi_draw_info, plotman_obj, more_commands    
   hxi_draw_info, plotman_obj, t_range=time_range,/spec
end

pro hxi_spectrum::setdata,data_str=data_str
    self.spec_str=ptr_new(data_str)
end

pro hxi_spectrum__define
  self = {hxi_spectrum, tstart:0.d0, tend:0.d0, time_interval:0., collnum:ptr_new(),$
    energy_bins:ptr_new(), data_units:0, spec_str:ptr_new(0)}
end
