;+
; Mission     : ASO-S/HXI
; Name        : hxi_draw_info_on_plot
;Purpose:   plot some information (HXI image/spectrum... params) on the plotman object window, performing specifically.
;Category:  auxiliary code for the ASO-S/HXI software
;
;Inputs:    
;           obj_pc   - a 'PLOTMAN_PLOT_CONTROL' structure from plotman__define, used here to judge some params, if needed
;           enebins  - energy range, e.g.，[10.,300], reserved parameters 
;           method   - imaging method with HXI, e.g., 'Vis_Clean',  reserved parameters 
;           counts_total  - counts for the HXI image, directly from the input .fits file for maps
;           counts_bg     -                   , directly from the input .fits file for maps
;           t_range  - date-time range string, e.g., '11-Nov-22 11:11:00.000 to 11:11:00.000',  reserved parameters
;           
;(keyword): spec     - add t_range info to the HXI spectrum plotting
;
;Called by:          - hxi_draw_info.pro or plotman object with more_commands if under plotman environments
;
;History:
;  2023/03/24, Fu Yu
;       04/01, add keyword_set spec for t_range
;       04/09, Yang Su, only show the integer value of the counts
;       04/11, Fu Yu, add obj_pc,overlay_tag, when using overlay keyword for multi-maps, the counts information is not displayed on the map(avoid confusion)
;  2024/02/04, Yang Su, add detid_mm for showing detector range on plot; all inputs are now strings.
;-

PRO hxi_draw_info_on_plot,obj_pc=obj_pc, enebins=enebins,method=method,$
                          counts_total=counts_total,counts_bg=counts_bg, detid_mm=detid_mm, $  ;im_info
                      t_range=t_range, $
                      spec=spec

; 1. prep params
params=[]  ;
values=[]  ;  

if keyword_set(enebins) then begin
  params=[params,'enebins:']
  values=[values,enebins]
endif

if keyword_set(method) then begin
  params=[params,'method:']
  values=[values,method]
endif
 
if keyword_set(counts_total) then begin
  params=[params,'Counts: ']
  values=[values, counts_total ]  ;230409, yang su, add fix();240204, YS, 
endif

if keyword_set(counts_bg) then begin
  params=[params,'BKG Counts:']
  values=[values,  counts_bg ];230409, yang su, add fix();240204, YS, 
endif

;240204, yang su 
if keyword_set(detid_mm) then begin
  params=[params,'Det. range:']
  values=[values, detid_mm ]
endif

if keyword_set(t_range) then begin
  params=[params,'']
;  values=[values,hxi_en2str(t_range,/tim)]
values=[values,t_range]
endif
;stop
overlay_tag=0
if keyword_set(obj_pc) then begin
  overlay_tag = max(strlen(obj_pc.overlay_panel)) + max(strlen(obj_pc.OVERLAY_ULABEL))
endif


; 2. plot setting
nparams=n_elements(params)
shifty=indgen(nparams)*0.05; [0,0.05] ; location shift

;plotman_obj->select
xlim = !x.crange & ylim = !y.crange
hl=ylim[1]-ylim[0] & wl=xlim[1]-xlim[0]
refx = xlim[0] & refy = ylim[1] ; left upper


if keyword_set(spec) and keyword_set(t_range) then begin
  
;   t_range = '11-Nov-22 11:11:00.000 to 11-Nov-22 11:11:00.000'  ; for test
  XYOUTS, 10^(refx+ 0.95*wl), 10^(refy-(0.06+ 0 )*hl), t_range, ALIGNMENT=1.
  
endif else begin
  if overlay_tag lt 1 then begin
    ;print,'overlaying info on the map...'
    for i=0, nparams-1 do begin
      ;XYOUTS, refx+0.05*wl, refy-0.05*hl, method+', '+enebins
      XYOUTS, refx+0.05*wl, refy-(0.06+shifty[i])*hl, params[i] +string(values[i]),charsize=1.2
    endfor
  endif ;overlay_tag
endelse


END; the .pro




