;+
; Mission     : ASO-S/HXI
; Name        : hxi_get_bkgdata_48h
; Purpose     : get bkg for imaging from data 48h before or ahead
; Category    : imaging
; Syntax      :
; Inputs      :
;       trange      - input time intervals, [2,1 ]
;       expt        - exptime array
;       earray      - input energy ranges [2, nenergy]
;       [dtime]     - time difference (-48h*3600s or 48h*3600s);
; Outputs     :
;       crate_bkg     - counts data for given time intervals and energy ranges
;       crate_bkgdet  - background data for given time interval and energy ranges
;       counts_bkg    - total counts, [n original time intervals]
;       counts_bkgdet - total bkg counts, [n original time intervals]
; History     :
;       2023/09/01, Yang Su, 
; Contact     : yang.su@pmo.ac.cn
;-

pro hxi_get_bkgdata_48h,trange, expt, earray,crate_bkg,crate_bkgdet,counts_bkg, $
              counts_bkgdet,totcounts_totdet_48h, totcounts_bkgdet_48h, dtime = dtime
  
  default, dtime, -3600.*48.
  this_trange = atime( anytim([trange[0], trange[-1]]) +[-expt[0], expt[-1]]/2. )
  expt_tot = anytim( this_trange[-1] ) - anytim( this_trange[0] )
  trange_bkg =  atime( anytim( this_trange )  + dtime  ) ;+[-4.1,4.1]
  
  hxi_get_counts4imaging,  earray, trange_bkg,  counts_str=counts_str, $
                    cent_xy=cent_xy, prod=prod, tcalib=1, filename=filename ;240107, YS, tcalib
 
   counts = counts_str.counts_subcoll
   counts_bkgdet = counts_str.counts_bkgdet
   counts_bkgdet_edge =counts_str.counts_bkgdet_edge
   tot_counts_bkg= counts_str.counts_totdet
   tot_counts_bkgdet= counts_str.counts_totdet_bkg
   tarray_bkg = counts_str.tarray_out
   expt_bkg = counts_str.expt
   livet = counts_str.livet_subcoll
   livet_tot = counts_str.livet_totdet
   livet_bg = counts_str.livet_bkgdet
   livet_bg_edge = counts_str.livet_bkgdet_edge
  
          crate_bkg  = fltarr(91,n_elements(expt) )
          crate_bkgdet  = fltarr(91,n_elements(expt) )
          
          totcrate_48h_totdet = fltarr( n_elements(expt) )
          totcrate_48h_bkgdet = fltarr( n_elements(expt) )
          
          dt_start = anytim(tarray_bkg[0])- (anytim( trange[0] )  + dtime) 
          
          for kk=0, 90 do begin
            crate_bkg[kk,*] = interpol( reform(counts[kk,*])/livet[kk,*] ,total(expt_bkg,/cumu)-expt_bkg/2. ,   total(expt,/cumu)-dt_start-expt/2. )
            crate_bkgdet[kk,*] = interpol( reform(counts_bkgdet[kk,*])/livet[kk,*] , total(expt_bkg,/cumu)-expt_bkg/2. ,  total(expt,/cumu)-dt_start-expt/2. )
            
          endfor
          
          totcrate_48h_totdet = interpol( (tot_counts_bkg)/livet_tot ,total(expt_bkg,/cumu)-expt_bkg/2. ,   total(expt,/cumu)-dt_start-expt/2. )
          totcrate_48h_bkgdet = interpol( (tot_counts_bkgdet)/livet_bg ,total(expt_bkg,/cumu)-expt_bkg/2. ,   total(expt,/cumu)-dt_start-expt/2. )
          
   counts_bkg = rebin( crate_bkg, 91, n_elements(expt),/sample)  * transpose( rebin(expt, n_elements(expt),91,/sample))
   counts_bkgdet = rebin( crate_bkgdet, 91, n_elements(expt),/sample)  * transpose( rebin(expt, n_elements(expt),91,/sample))
   
   totcounts_totdet_48h =  ( totcrate_48h_totdet * expt )
   totcounts_bkgdet_48h =  ( totcrate_48h_bkgdet * expt )
 
end