;+
; Mission     : ASO-S/HXI
; Name        : hxi_image_params_event
; Purpose     : Build composite components that enable users to select image parmeters
;input:
; parent - widget id of widget calling hxi_image_params
;Outputs:
; uvalue - user value to assign to compound widget
;History:
; Writen by Fanxiaoyu Xia, 2022/07/16
; 2021/09/21, Fanxiaoyu Xia, updated the values immediately after modification. If the values
;   are ' ', then return error message and modified them back to default values. If the values 
;   are '', then set them to '-10000'
; 2023/04/09, Yang Su, changed deafult setting of parameters
; 2023/12/22, Yang Su, Changxue Chen, add img_calib

function hxi_image_params_event, event
  base = event.handler
  stash = widget_info(base, /child)
  widget_control, stash, get_uvalue=state
  widget_control, event.id, get_uvalue=uvalue
  
  case uvalue of
    'xc': begin
      widget_control,state.text1,get_value=xcenter
      if xcenter ne ' ' then begin                     ;@YF 2023 03 30
        if xcenter eq '' or xcenter eq '-' then begin  ;@YF 2023 03 30
          state.xc=-10000
        endif else begin
          state.xc=float(xcenter)
        endelse
      endif else begin
        a = dialog_message('X-Center is empty!')
        widget_control,state.text1,set_value=state.cparams[0]
        state.xc=float(state.cparams[0])
      endelse
    end

    'yc': begin
      widget_control,state.text2,get_value=ycenter
      if ycenter ne ' ' then begin                     ;@YF 2023 03 30
        if ycenter eq '' or ycenter eq '-' then begin  ;@YF 2023 03 30
          state.yc=-10000
        endif else begin
        state.yc=float(ycenter)
        endelse
      endif else begin
        a = dialog_message('Y-Center is empty!')
        widget_control,state.text2,set_value=state.cparams[1]
        state.yc=float(state.cparams[1])
      endelse
    end

    'nxy': begin
      widget_control,state.text3,get_value=nxy
      if nxy ne ' ' then begin
        if nxy eq '' then begin
          state.nxy=-10000
        endif else begin
        state.nxy=float(nxy)
        endelse
      endif else begin
        a = dialog_message('nxy is empty!')
        widget_control,state.text3,set_value=state.cparams[2]
        state.nxy=float(state.cparams[2])
      endelse
    end

    'psize': begin
      widget_control,state.text4,get_value=psize
      if psize ne ' ' then begin
        if psize eq '' then begin
          state.psize=-10000
        endif else begin
        state.psize=float(psize)
        endelse
      endif else begin
        a = dialog_message('psize is empty!')
        widget_control,state.text4,set_value=state.cparams[3]
        state.psize=float(state.cparams[3])
      endelse
     end
      
         ;231222,YS,CC   
      'calib_control': begin
          calib_state = event.select
          ;widget_control,state.w_nocalib,set_button= calib_state
          if calib_state eq 0 then begin
            *state.img_calib=0
          endif else begin
            *state.img_calib=1
          endelse
        end
    
  end
  widget_control, stash, set_uvalue=state
  ret = {id: base, top: event.top, handler: 0L, calib_state:*state.img_calib}   ;231222,YS,CC   
  return,ret
end

function hxi_image_params, parent, uvalue=uvalue
  w_base = widget_base ( parent, $
    event_func='hxi_image_params_event', /base_align_left, $
    ;/column, uvalue=uvalue, y_scroll_size=150, $
    /column, uvalue=uvalue, $
    title='Collimator/Detector Options', $
    /frame)
  cparams=['409.','280.','101','2']
  base1 = widget_base(w_base, /row, space=32)
  labe1 = widget_label(base1, value='X-Center (arcsec): ')
  text1 = widget_text(base1,value=cparams[0], xsize=6, /editable,/all_events,uvalue='xc')
  base2 = widget_base(w_base, /row, space=32)
  labe2 = widget_label(base2, value='Y-Center (arcsec): ')
  text2 = widget_text(base2,value=cparams[1], xsize=6, /editable,/all_events,uvalue='yc')

  labe3 = widget_label(base1, value='Nxy:   ')
  text3 = widget_text(base1,value=cparams[2], xsize=6, /editable,/all_events,uvalue='nxy')
  labe4 = widget_label(base2, value='Psize: ')
  text4 = widget_text(base2,value=cparams[3], xsize=6, /editable,/all_events,uvalue='psize')
  
    ;231222,YS,CC   
  ;----------------------------
    w_calib1 = widget_base(base1, /nonexclusive, /row, space=8., xpad=0 )
    
    default, img_calib, 0
    w_nocalib = widget_button( w_calib1, $
      value='calib. test', uvalue='calib_control')
    widget_control, w_nocalib, set_button = img_calib
  ;----------------------------------------------
  
  state= { w_base: w_base, $
    text1:text1,text2:text2,text3:text3,text4:text4,$
    xc:409.,yc:280.,nxy:101,psize:2.,img_calib:ptr_new(img_calib),w_nocalib:w_nocalib, cparams:cparams}
 
  widget_control, widget_info(w_base, /child ), set_uvalue=state
  return, w_base
end
