;+
; Mission:
;   ASO-S/HXI
; Purpose:
;     Calculate the diagonal detector response matrix of 
;     HXI user-defined energy bands
; Inputs:
;    conv     --   Detector response matrix of HXI
;    ph_ener  --   Array of energy edges of photons in HXI simulated drm
;    ph_edges --   Array of energy edges of photons in User defined drm
; Keyword:
;     linear  --   The distribution of energy intervals in the drm
;     power   --   default mode, p=-3.5
; Output:
;     uconv   --   Calculated conversion factor for user
; History:
;     WeiChen,   2022/01/24


pro hxi_extract_diag4user, conv, ph_ener, ph_edges, $
                           uconv, linear=linear,power=power

  if n_params() NE 4 then Message, 'Incorrect number of arguments.'
  
  power = fcheck(power,-3.5)
  if min(ph_edges) lt min(ph_ener) OR max(ph_edges) gt max(ph_ener) then $
    message, 'The energy bands set is beyond range! ',/info
    
    em = ph_ener
  edge_products,ph_edges,edges_1=ph_edge1, edges_2=ph_edge2, mean=ephm,width=ephw
    nph= n_elements(ephw)
    uconv= dblarr(nph)
  
 indy= closest_ix(em, ph_edge1)  
   if keyword_set(linear) then begin
    for i=0, nph-1 do uconv(i) = f_div(total( conv(indy[i]:indy[i+1]-1)), indy[i+1]-indy[i])
  endif else begin
    weight= em ^ power
    for i=0, nph-1 do  uconv(i) = f_div( total( conv(indy[i]:indy[i+1]-1)* weight(indy[i]:indy[i+1]-1)), $
        total( weight(indy[i]:indy[i+1]-1)) )
  endelse
   


END