;+
; Mission:
;  ASO-S/HXI
; Purpose:
;  Get user drm from G4 simulation results
; Inputs:
;  out_ener --  The deposit energy bands of output spectrum
; Outputs:
;  udrm     --  DRM for user
;  pho_ener
; Op. Inputs:
;  pho_ener --  The incident energy bands of spectrum
;  drmfile  --  Externally supplied drm
;  linear   --  The distribution of energy intervals in the drm
;  thin_det --  DRM for thin window detector
;  diagonal --  Only extract diagonal lines of the DRM
; Example:
;  hxi_get_srm4user, out_ener, udrm, pho_ener=pho_ener,/linear,/diagonal
; History
;  Weichen,  2022/01/16
;  Weichen,  2024/01/25, add keyword pse, pulse-spread effect,
 

PRO hxi_get_srm4user, out_ener, udrm, pho_ener=pho_ener, $
       drmfile=drmfile,linear=linear, thin_det=thin_det, diagonal=diagonal,power=power,$
       pse = pse
 
  ;checkvar,out_ener, 4
  delim   = PATH_SEP() 
  
  
;  drmfile= KEYWORD_SET( drmfile)? drmfile: GETENV('hxi_spec')+delim+'resp_v7_0p85mm_no.sav'  ;230519, YS, lower case
;   IF KEYWORD_SET(THIN_DET) THEN  drmfile=  GETENV('hxi_spec')+delim+'resp_v7_0p6mm_no.sav'  ;230519, YS, lower case
 
  drmfile= KEYWORD_SET( drmfile)? drmfile: $
      KEYWORD_SET(THIN_DET)? GETENV('hxi_spec')+delim+'resp_v9_0p50mm.sav' : $
        GETENV('hxi_spec')+delim+'resp_v9_0p85mm.sav'  ; 2024/01/31 chen; 24/02/06, lzt
 
   ;print, drmfile
   restore, drmfile  ;ph_edges,ct_edges,eff,trans,resp
   
   
    sz=SIZE(resp,/dimension)
    ph_mean=  N_ELEMENTS(ph_edges) EQ sz(1)
 
   ph_edge1= ph_mean ? ph_edges : get_edges(ph_edges, /edges_1)
   edge_products, ct_edges, mean=ctm, edges_1= ct_edge1, width= ctw

  ;;user energy band
  out_ener= get_edges(out_ener,/edges_1)
  pho_ener=KEYWORD_SET(pho_ener) ? get_edges(pho_ener,/edges_2): hxi_srm_extend_ph_edges(out_ener)  
  
  ;2024/01/25 weichen
    orgresp = resp
  IF KEYWORD_SET(pse) THEN BEGIN
    ps_input = { ein:get_edges([ph_edges[0],ph_edges],/edges_2), eout:get_edges(ct_edges,/edges_2), func:'hxi_fwhm', func_par:1  }
    pulse_spread, ps_input, pulse_shape,orgresp, resp
  ENDIF

  IF KEYWORD_SET(DIAGONAL) THEN BEGIN
    conv = MAX(orgresp,dim=1)
    hxi_extract_diag4user, conv, ph_edge1, pho_ener, udrm, linear=linear,power=power
  ENDIF ELSE BEGIN
    hxi_extract_srm4user,resp, ct_edge1,ph_edge1, out_ener, pho_ener, udrm, /perbin,ph_mean=ph_mean,linear=linear, power=power
   
  ENDELSE

 
  end
  
  
  
