;+
; Mission:
;    ASO-S/HXI
; Purpose:
;    Adjust the exposure time of the energy spectrum
; Inputs:
;    spec  --    struct include time,exptime,counts & ltime. 
; Keyword:
;    timebin  --  units of second
;    hxistyle --  hxi data style
; Output:    
;    spec  --     new spec
; History:
;     WeiChen, 2022/11/16
;              2023/04/16 add HXI data style
;              2024/01/30 fixed a bug for last timebin
;
FUNCTION HXI_REBIN_EXPTIM2,specin,timebin=timebin, hxistyle=hxistyle

on_error,2
checkvar,timebin, 1.


  oldspec = specin
  spec = specin

if keyword_set(hxistyle) then begin
 
  szc= size(specin.rcounts,/dim)

  info={ttime:0d, exptime:0. ,$
    counts: fltarr(szc[0],szc[1]), $
    ltime: fltarr(szc[1]) }
  spe = replicate(info,szc[2])
  spe.ttime = specin.ut_edges[0]
  spe.exptime =get_edges(specin.ut_edges, /width)
  spe.counts = specin.rcounts
  spe.ltime = reform(specin.ltime[0,*,*])
  
  spec=spe
  
endif 

  ttime  = spec.ttime
  exptime= spec.exptime
  counts = spec.counts
  ltime  = spec.ltime



ind = where(exptime lt 0.4,nt);确认处于爆发模式的帧数，可能非连续
  IF nt EQ 0 THEN RETURN, oldspec  ;No merge time is required

piece = find_contig(ind,is1,is2)
npiece= n_elements(piece) ;爆发段数
if npiece gt 1 then print, 'There are '+trim(npiece)+' triggers in the data'
if timebin LT min(exptime) then begin
  MESSAGE, 'Timebin set must great than burst exptime.',/info
  RETURN, oldspec
endif
;新的曝光时间里合并的时间段数量
mbin_tk = fix(timebin/min(exptime))

;=======pattern 1=========
;一维数组光变  ntime
IF (SIZE(COUNTS))[0] EQ 1 THEN GOTO, SINGLEPATTERN

;=======pattern 2==========
;三维数组光变  [neband, ndet, ntime]
;;;如果只有一个连续的爆发模式,大多数情况如此
IF NPIECE EQ 1 THEN BEGIN
  ;新合并后的时间数目
  mbins = nt/mbin_tk
  ;剩余未合并的数量
  mod_bins = nt MOD mbin_tk

  org_exptime = (exptime[ind])[0:nt-1-mod_bins]
  new_exptime = total(reform(org_exptime, mbin_tk, mbins),1)
   if mod_bins NE 0 then new_exptime= [new_exptime, total(org_exptime[-mod_bins:*],1)]


  counts_rebin= counts[*,*,ind]
  ltime_rebin = ltime[*,ind]
   sz=size(counts_rebin,/dim)

  newcounts= total(reform(counts_rebin[*,*,0:nt-1-mod_bins], sz[0], sz[1], mbin_tk,mbins),3)
    if mod_bins EQ 1 then newcounts= [[[newcounts]],[[reform(counts_rebin[*,*,-mod_bins],sz[0],sz[1],1)]]]
    if mod_bins GT 1 then newcounts= [[[newcounts]],[[reform(total(counts_rebin[*,*,-mod_bins:*],3),sz[0],sz[1],1)]]]
  newltime = total(reform(ltime_rebin[*,0:nt-1-mod_bins], sz[1], mbin_tk,mbins),2)
    if mod_bins EQ 1 then newltime = [[newltime], [reform(ltime_rebin[*,-mod_bins], sz[1],1)]]
    if mod_bins GT 1 then newltime = [[newltime], [reform(total(ltime_rebin[*,-mod_bins:*],2), sz[1],1)]]

  org_ttime= (ttime[ind])[0:nt-1-mod_bins]
  new_ttime= org_ttime[0:*:mbin_tk]
   if mod_bins NE 0 then new_ttime=[new_ttime,ttime[ind[-mod_bins]]]   ;;;weichen,2023/12/25

  ;假设没有多余时间段未合并；没有间断的ind
  ;2024/01/30 weichen
  if ind[0] NE 0 then begin
    if ind[-1] NE n_elements(ttime)-1 then begin
      rebuild_time  = [ttime[0:ind[0]-1], new_ttime, ttime[ind[-1]+1:*]]
      rebuild_counts= [[[counts[*,*,0:ind[0]-1]]],[[newcounts]],[[counts[*,*,ind[-1]+1:*]]]]
      rebuild_ltime = [[ltime[*,0:ind[0]-1]], [newltime], [ltime[*,ind[-1]+1:*]]]
    endif else begin
      rebuild_time  = [ttime[0:ind[0]-1], new_ttime]
      rebuild_counts= [[[counts[*,*,0:ind[0]-1]]],[[newcounts]]]
      rebuild_ltime = [[ltime[*,0:ind[0]-1]], [newltime]]
    endelse
  endif else begin
    if ind[-1] NE n_elements(ttime)-1 then begin
      rebuild_time  = [ new_ttime, ttime[ind[-1]+1:*]]
      rebuild_counts= [[[newcounts]],[[counts[*,*,ind[-1]+1:*]]]]
      rebuild_ltime = [[newltime], [ltime[*,ind[-1]+1:*]]]
    endif else begin
      rebuild_time  = new_ttime
      rebuild_counts= newcounts
      rebuild_ltime = newltime
    endelse
  endelse
  
  ;;2023/04/16
  rebuild_exptime = get_edges([rebuild_time,last_item(ttime)+last_item(exptime)],/width)
  
  szc= size( rebuild_counts,/dim)
  
  info={ttime:0d,$
        exptime: 0., $
        counts: fltarr(szc[0],szc[1]), $
        ltime:  fltarr(szc[1]) }
  outspec = replicate(info,n_elements(rebuild_time))
    outspec.ttime  = rebuild_time
    outspec.exptime= rebuild_exptime
    outspec.counts = rebuild_counts
    outspec.ltime  = rebuild_ltime
  

ENDIF ELSE BEGIN
  spec1  = spec[0:ind[is2[0,1]]]
  outspec= HXI_REBIN_EXPTIM2(spec1,timebin=timebin)
  for ti =1, npiece-1 do begin
    ;;爆发片段的总数目
    tempspec= spec[ind[is2[ti-1,1]]+1: ind[is2[ti,1]]]
    outtemp = HXI_REBIN_EXPTIM2(tempspec,timebin=timebin)
    outspec = merge_struct(outspec,outtemp)
  endfor
  ;合并结尾未爆发时段
  if ind[-1] NE n_elements(spec)-1 then $
     outspec= merge_struct(outspec, spec[ind[-1]+1:*])
  
ENDELSE

if keyword_set(hxistyle) then begin
  spe= outspec
  ntime= n_elements(spe)
  sz= size(spe.counts,/dim)
  
  info= {ut_edges:[0.d,0.d], $
    ct_edges:oldspec[0].ct_edges, $
    rcounts: fltarr(sz[0],sz[1]),  $
    ltime: fltarr(sz[0],sz[1]) }


  outspec = replicate(info,ntime)
    outspec.ut_edges = get_edges([spe.ttime,last_item(spe.ttime)+last_item(spe.exptime)],/edges_2)
    outspec.ct_edges = reproduce(oldspec[0].ct_edges,ntime)
    outspec. rcounts = spe.counts
    outspec.ltime    = transpose(reproduce(spe.ltime,sz[0]),[2,0,1])
  
endif

spec = oldspec
RETURN,outspec

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
SINGLEPATTERN:
;;;如果只有一个连续的爆发模式,大多数情况如此
IF NPIECE EQ 1 THEN BEGIN
  ;新合并后的时间数目
  mbins = nt/mbin_tk
  ;剩余未合并的数量
  mod_bins = nt MOD mbin_tk

  org_exptime = (exptime[ind])[0:nt-1-mod_bins]
  new_exptime = total(reform(org_exptime, mbin_tk, mbins),1)
    if mod_bins NE 0 then new_exptime= [new_exptime, total(org_exptime[-mod_bins:*],1)]


  counts_rebin= counts[ind]
  ltime_rebin = ltime[ind]
  
  newcounts = total(reform(counts_rebin[0:nt-1-mod_bins], mbin_tk,mbins),1)
    if mod_bins NE 0 then newcounts= [newcounts,total(counts_rebin[-mod_bins:*])]
  newltime  = total(reform(ltime_rebin[0:nt-1-mod_bins], mbin_tk,mbins),1)
    if mod_bins NE 0 then newltime = [newltime, total(ltime_rebin[-mod_bins:*])]

  org_ttime = (ttime[ind])[0:nt-1-mod_bins]
  new_ttime = org_ttime[0:*:mbin_tk]
   if mod_bins NE 0 then new_ttime=[new_ttime,ttime[ind[-mod_bins]]]

  ;假设没有多余时间段未合并；没有间断的ind
  if ind[0] NE 0 then begin
    if ind[-1] NE n_elements(ttime)-1 then begin
      rebuild_time  = [ttime[0:ind[0]-1], new_ttime, ttime[ind[-1]+1:*]]
      rebuild_counts= [counts[0:ind[0]-1],newcounts,counts[ind[-1]+1:*]]
      rebuild_ltime = [ltime[0:ind[0]-1], newltime, ltime[ind[-1]+1:*]]
    endif else begin
      rebuild_time  = [ttime[0:ind[0]-1], new_ttime]
      rebuild_counts= [counts[0:ind[0]-1],newcounts]
      rebuild_ltime = [ltime[0:ind[0]-1], newltime]
    endelse
  endif else begin
    if ind[-1] NE n_elements(ttime)-1 then begin
      rebuild_time  = [new_ttime, ttime[ind[-1]+1:*]]
      rebuild_counts= [newcounts,counts[ind[-1]+1:*]]
      rebuild_ltime = [newltime, ltime[ind[-1]+1:*]]
    endif else begin
      rebuild_time  = new_ttime
      rebuild_counts= newcounts
      rebuild_ltime = newltime
    endelse   
  endelse

  ;;2023/04/16
  rebuild_exptime = get_edges([rebuild_time,last_item(ttime)+last_item(exptime)],/width)

  info={ttime:0d,$
        exptime:0. ,$
        counts: 0., $
        ltime:0.}
  outspec = replicate(info,n_elements(rebuild_time))
    outspec.ttime  = rebuild_time
    outspec.exptime= rebuild_exptime
    outspec.counts = rebuild_counts
    outspec.ltime  = rebuild_ltime


ENDIF ELSE BEGIN
  spec1= spec[0:ind[is2[0,1]]]
  outspec= HXI_REBIN_EXPTIM(spec1,timebin=timebin)
  for ti =1, npiece-1 do begin
    ;;爆发片段的总数目
    tempspec= spec[ind[is2[ti-1,1]]+1: ind[is2[ti,1]]]
    outtemp = HXI_REBIN_EXPTIM(tempspec,timebin=timebin)
    outspec = merge_struct(outspec,outtemp)
  endfor
 ;合并结尾未爆发时段
  if ind[-1] NE n_elements(spec)-1 then $
     outspec= merge_struct(outspec, spec[ind[-1]+1:*])

ENDELSE


spec= oldspec
RETURN, OUTSPEC

END