;+
; Mission:
;   ASO-S/HXI
; Purpose:
;   Calculate initial phase and amplitude of triangular wave of 
;     grid pairs with phase difference of 120 degrees
;  
; Input:
;  counts of triple-grids (c1,c2,c3)
; Op. input:
;  DC  -- DC component
; Ouput:
;  obsvis value for sin sub-collimator
; Op. output
;  amp   --  amplitude of visibility
;  phase --  phase of visibility
;  
; History
;  Weichen, 2022/08/09

FUNCTION HXI_CAL_TRI_COLLIMATOR, obs, amp=amp, phase=phase,dc=dc

; IF N_ELEMENTS(obs) NE 3 THEN BEGIN
;  MESSAGE, 'Sytrax: hxi_cal_tri_collimator([o1,o2,o3])'
;  RETURN, -1
; ENDIF

  ;Assume that the DC component is 1/4 of the total flow; Or through this set of grids
  y1=obs[0,*] & y2=obs[1,*] & y3=obs[2,*]  ;yang, 20220925
  dc= is_number(dc)? dc: (y1+y2+ y3)/3.
  
  tmp1 = FLOAT( y3-y2)
  tmp2 = FLOAT( y1-dc)
  amp  = SQRT( 1/3.*tmp1^2 + tmp2^2 )
  phase = ATAN( tmp1/tmp2/SQRT(3))
  
  
  ;The count of the first set of triangular waves acts as a cosine collimator,
  ;  and returns a hypothetical sine collimator count
  result= (y3- y2)/SQRT(3) + dc

  RETURN, RESULT


END

;stop
;;test
;x=findgen(100)*0.1
;amp=5. 
;f0= 20
;a=amp*cos(x)+f0
;phi0 = 20*!dtor
;a1=amp*cos(phi0) +f0
;a2=amp*cos(phi0+120*!dtor)+f0
;a3=amp*cos(phi0+240*!dtor)+f0
;
;a4=amp*cos(phi0- 90*!dtor) +f0
;
;obs=[a1,a2,a3]
;r = hxi_cal_tri_collimator(obs)
;
;END