;+
; Mission     : ASO-S/HXI
; Name        : bkg_help_widget
; PURPOSE     : Show help for selecting background data.
; HISTORY:
;  Writen by Changxue Chen, 2023/10/17
;  2023/10/20, Changxue Chen, modify the window size based on the current resolution of the screen.

pro bkg_help_widget,group_leader=group_leader
  device,get_screen_size=sc
  xsize = sc[0]*0.7  ;231230,YS
  ysize = sc[1]*0.6
  btitle = 'Help of Selecting Background Data'
  modal=0
  w_base = widget_base(TITLE = btitle, /colum,/frame, modal=modal, group_leader=group_leader,xsize=xsize,ysize=ysize)
  
  ;231230,YS
  value = ['This is a brief help on how to select background data. ' ,$
    '                       ',$
    'Firstly, you need to select the type of background data, including:',$
    '            ',$
    '  1, Type-1 represents the current BKG detector data (average of D95,D96 and D99),',$  
    ' which is the default background data, but is not recommended in most cases.',$
    ' It may work when the background levels are quite low. ',$
    '              ',$
    '  2, Type-2 represents the fitting results of data for non-flare time periods, ',$
      'which requires us to select pre/post flare data as the background and perform the fitting.',$
      ' There are currently three fitting methods available: "0Poly","1Poly","BKG_profile".', $
      '            ',$  
     '  3, Type-3 represents the corresponding MODULATION detector data before/after 48 hours.',$
      
      '                        ',$
      'Secondly, you can click the "Plot Data" button to view the profile of the selected',$
      '   background data for the selected imaging time range.',$
      '                 ' ,$
      'Finally, you can click the "Display Counts of Different Detectors" button ', $
      '   to view the count of each detector and the distribution of incident photons.',$
      '              ',$
      ' note1: Type-1 shall be used only when there is no better choice. ',$ 
      ' note2: BKG_Profile shall be used with caution as the BKG of different detectors are in fact different.',$
      '        BKG_Profile works best when the BKG level is low.',$
      ' note3: Be aware that Type-2 (0poly and 1poly) may not represent the actual BKG profile.',$   
      ' note4: Type-3 can be used if there is no particle or flare event 48h before or ahead ',$
      '             and the level of background is consistent with current level.',$ 
      ' note5: BKG selection for HXI is a case-by-case job. Sorry about that:):( ',$  
      ' note6: Currently (v1.40beta), visibilities have not been calibrated.',$
      '         But vis-based methods (vis_clean) may be used now (better without G1 and G2).',$
      '         vis_bp and vis_clean are fast in gerneral.',$
      ' note7: Currently (v1.40beta), "calibration test" is only testing for pattern-based methods (slow!).', $
      ' note8: Working on a fast version of HXI_clean.' ] 
      
  w_help = WIDGET_TEXT(w_base,value = value,/WRAP,xsize=xsize,ysize=ysize)
  widget_control,w_base,/realize

end