;+
; Mission     : ASO-S/HXI
; Name        : bkg_widget
; Purpose     : Open a window that enable the user to select different bkg data for imaging
; Category    : data processing and production; imaging
; Syntax      :
; Inputs      :
;       trange      - time periods for imaging [2,n]
;       erange      - energy ranges for imaging [2,n]
;       bkg_mode    - 1/2/3, different mode represent different bkg data
;                   Data 1 represents the current BKG detector data (default)
;                   Data 2 represents the fitting results of non-flare data 
;                   Data 3 represents MODU detector data before/after/average 48 hours
;                   
;       bkg_obj     - 
;       plotman_obj -
;
; Outputs     :
;       bkg_str     - a struct including all data for imaging
;       
; History     :
;       2023/09/20, Changxue Chen, the first version
;       2023/10/17, Changxue Chen, migrate the current background selection program to the HXI main program,
;          which is no longer used.

pro bkg_widget_event, event
  widget_control,event.top,get_uvalue = state
  widget_control,event.id,get_uvalue = uvalue

  tarray =  state.tarray ;ut format
  earray = state.earray ;array [2,n]
  ty =state.ty
  bkg_obj = state.bkg_obj

  case uvalue of
    'PLOT1':begin
      ;--------------------------------------
      bkg_str =bkg_obj. getdata( time_range=tarray, bkgtype='bkg_det',e_range=earray)  
      bkg_obj->plotman, bkg_str = bkg_str, plotman_obj=state.plotman_obj, time_range =tarray,$
        bkgtype='bkg_det',e_range=earray
    end

    'PLOT2':begin
      bkg_str =bkg_obj. getdata( time_range=tarray, bkgtype='fit_curve',e_range=earray,fitmode =  *state.fit_mode, $
        fit_trange = atime(*state.fit_tr))
      bkg_obj->plotman, bkg_str = bkg_str, plotman_obj=state.plotman_obj, time_range =tarray,$
        bkgtype='fit_curve',e_range=earray
    end

    'PLOT3':begin
      bkg_str =bkg_obj. getdata( time_range=tarray, bkgtype='48h_modu_det',e_range=earray, bkgmode= *state.bkg_type)
      bkg_obj->plotman, bkg_str = bkg_str, plotman_obj=state.plotman_obj, time_range =tarray,$
        bkgtype='48h_modu_det',e_range=earray
    end

    'DATA1':begin
      *state.info_data= 'Now is data-1  '
      *state.bkg_mode = 1
      bkg_str =bkg_obj. getdata( time_range=tarray, bkgtype='bkg_det',e_range=earray)
      *state.bkg_str=bkg_str
      message,/info,'Set data-1 as BKG-data'
    end
    'DATA2':begin
      *state.info_data= 'Now is data-2  '
      *state.bkg_mode = 2
      bkg_str =bkg_obj. getdata( time_range=tarray, bkgtype='fit_curve',e_range=earray,fitmode =  *state.fit_mode, $
        fit_trange = atime(*state.fit_tr))
        
      *state.bkg_str=bkg_str
      message,/info,'Set data-2 as BKG-data'
    end
    'DATA3':begin
      *state.info_data= 'Now is data-3  '
      *state.bkg_mode = 3
      bkg_str =bkg_obj. getdata( time_range=tarray, bkgtype='48h_modu_det',e_range=earray, bkgmode= *state.bkg_type)
      *state.bkg_str=bkg_str
      message,/info,'Set data-3 as BKG-data'
    end
    'nothing':begin
      numTabs = widget_info(state.wtab, /tab_number)
    end
    'BKG_TYPE':begin
      widget_control,state.bkg_button,get_value = b_type ;0 / 1
      if b_type eq 0 then *state.bkg_type = -1
      if b_type eq 1 then *state.bkg_type = 1
      widget_control,event.top, set_uvalue=state
    end
;    'FIT_TIME':begin
;      intervals = atime(*state.fit_tr)
;      state.plotman_obj->intervals,intervals=intervals
;      ;stop
;      tim_arr = state.plotman_obj->get(/intervals) ;[2, ntim]
;      ;stop
;      tim_arr_str = atime(tim_arr)
;      tim_arr_str = strjoin(tim_arr_str,' to ')
;      tim_tmp = hxi_time_intervals_trans(tim_arr_str,/str)
;      *state.fit_tr=tim_tmp.t12 ; anytim-sec
;      widget_control, state.fit_times,set_value=tim_tmp.t12_str
;      widget_control, event.top, set_uvalue=state
;      end
    'FIT_METHOD':begin
      widget_control,state.fit_button,get_value = f_type ;0 / 1 /2 
      if f_type eq 0 then *state.fit_mode = '0Poly'
      if f_type eq 1 then *state.fit_mode = '1Poly'
      if f_type eq 2 then *state.fit_mode = '2Poly'
      ;if f_type eq 3 then *state.fit_mode = 'This E profile'
      
      widget_control,event.top, set_uvalue=state
      end
    else:
  endcase
  if uvalue eq 'QUIT' then widget_control,event.top,/destroy
end

pro bkg_widget,info_data=info_data,trange = trange,erange =erange,bkg_mode=bkg_mode, $
  bkg_trange = bkg_trange,bkg_obj=bkg_obj,plotman_obj=plotman_obj,bkg_str = bkg_str, $
  group_leader = group_leader

  if not have_windows() then begin
    message,'widgets are unavailable'
    return
  endif
  ;default setting-----------

  chars = os_family() eq 'unix' ? 1.3 : 1
  device,get_screen_size=sc
  fxpad = .0117*sc(0)
  xsize =  0.3*sc(0) &  ysize =  0.5*sc(1)
  space =20 ;tmp set
  
  if not keyword_set(info_data) then info_data = 'Now is data-1 '
  if not keyword_set(bkg_mode) then  bkg_mode= 1
  fit_mode = '0Poly'
  ; bkg_data = 0L
  tr_str = reform(trange[0,*] +' to '+trange[1,*]) ;trange is UT format
  fit_tr = bkg_trange
  fit_tr_str = atime(bkg_trange[0,*])+' to '+atime(bkg_trange[1,*])
  default,bkg_str,{}
  ;--------------------
  btitle = ' WIDGET SELECT BKG DATA FOR IMAGING '
  modal=0
  w_base = widget_base(TITLE = btitle, /row,/frame, modal=modal, group_leader=group_leader)
  base0 =  widget_base(w_base, /frame)
  ;base_right =  widget_base(w_base, /frame)
  wtab = widget_tab(base0, location=location, uvalue='nothing')
  ;wdraw = widget_draw(base_right,xsize=xsize ,ysize=ysize)
  ;--DATA1-------------------------
  wt1 = widget_base(wtab, title='BKD DATA-1', /column,space= space) ;,space= space, xsize =, ysize =
  lab1_1 =widget_label(wt1, value='Data 1 represents the current BKG detector data (default)',/ALIGN_LEFT)
  t_base1 = widget_base(wt1,/row )
  lab1_2 = widget_label(t_base1, value='Image Time Range:')
  w_times1 = widget_droplist(t_base1,value=tr_str, uvalue='nothing')

  e_base1 = widget_base(wt1,/row)
  lab1_3 = widget_label(e_base1, value='Image Energy Bins (keV):')
  w_energies1 = widget_droplist(e_base1, value=erange, uvalue='nothing')

  base1_0 = widget_base(wt1,/row,/frame)
  lab1_4 = widget_label(base1_0, value='Using the average data of BKG detectors D95, D96 as the BKG data')

  base1_1 = widget_base(wt1,/row)
  tmp = widget_button(base1_1,value='      PLOT DATA      ', $
    uvalue='PLOT1', /no_release)
  check1 = widget_button(base1_1,value='      SELECT DATA-1      ', $
    uvalue='DATA1', /no_release)
  quit = widget_button(base1_1,value='      QUIT      ',uvalue='QUIT', $
    /no_release)

  ;-DATA2--------------------------------
  wt2 = widget_base(wtab, title='BKG DATA-2', /column,space= space)
  lab2_1 =widget_label(wt2, value='Data 2 represents the fitting results of non-flare data ',/ALIGN_LEFT)
  
  t_base2 = widget_base(wt2,/row )
  lab2_2 = widget_label(t_base2, value='Image Time Range:')
  w_times2 = widget_droplist(t_base2,value=tr_str, uvalue='nothing')

  e_base2 = widget_base(wt2,/row)
  lab2_3 = widget_label(e_base2, value='Image Energy Bins (keV):')
  w_energies2 = widget_droplist(e_base2, value=erange, uvalue='nothing')

  base_fitting =  widget_base(wt2,/column,/frame)
  base2_0 = widget_base(base_fitting,/row)
  lab2_4 = widget_label(base2_0, value='Select the fitting range (non-flare intervals): ')
  ;button2_1 = widget_button(base2_0,value = ' Select with mouse ',uvalue ='FIT_TIME')

  fit_times = widget_droplist(base_fitting,value=fit_tr_str, uvalue='nothing')
  
  base2_2 = widget_base(wt2,/row,/frame)
  lab2_5 = widget_label(base2_2, value='Fitting Method: ')
  method_list = ['0Poly','1Poly','2Poly'];,'This E profile']
  fit_button = cw_bgroup(base2_2,method_list,/row,/exclusive,uvalue='FIT_METHOD',space=30)
  WIDGET_CONTROL, fit_button, set_value=0
  
  base2_1 = widget_base(wt2,/row)
  tmp = widget_button(base2_1,value='        PLOT DATA       ', $
    uvalue='PLOT2', /no_release)
  check2 = widget_button(base2_1,value='      SELECT DATA-2      ', $
    uvalue='DATA2', /no_release)
  quit = widget_button(base2_1,value='      QUIT      ',uvalue='QUIT', $
    /no_release)

  ;--DATA3-------------------------------------
  wt3 = widget_base(wtab, title='BKG DATA-3', /column,space= space)
  lab3_1 =widget_label(wt3, value='Data 3 represents MODU detector data before/after 48 hours ',/ALIGN_LEFT)
  t_base3 = widget_base(wt3,/row )
  lab3_2 = widget_label(t_base3, value='Image Time Range:')
  w_times3 = widget_droplist(t_base3,value=tr_str, uvalue='nothing')

  e_base3 = widget_base(wt3,/row)
  labe3_3 = widget_label(e_base3, value='Image Energy Bins (keV):')
  w_energies3 = widget_droplist(e_base3, value=erange, uvalue='nothing')

  base3_0 = widget_base(wt3,/row,/frame)
  labe3_4 = widget_label(base3_0, value='Select BKG Data Time Range:')
  type_list = ['Before 48h','After 48h']; 'Average 48h'
  bkg_button = cw_bgroup(base3_0,type_list,/row,/exclusive,uvalue='BKG_TYPE',space=30)
  bkg_type = -1 ;index -1-before 48h; 1-after 48h; 2-average 48h
  WIDGET_CONTROL, bkg_button, set_value=0


  base3_1 = widget_base(wt3,/row)
  tmp = widget_button(base3_1,value='      PLOT DATA      ', $
    uvalue='PLOT3', /no_release)
  check3 = widget_button(base3_1,value='      SELECT DATA-3      ', $
    uvalue='DATA3', /no_release)
  quit = widget_button(base3_1,value='      QUIT      ',uvalue='QUIT', $
    /no_release)
  ;-------------------------------------------------------------

  earray = unformat_intervals(erange)
  tarray = [atime(anytim(trange[0,0])),atime(anytim(trange[1,-1]))]

  en = n_elements(erange)
  ;stop
  ;xshow, group_leader
  widget_control,w_base,/realize
;  hsi_linecolors
  ty = 0.035

  state = {w_base:w_base,info_data:ptr_new(info_data),wtab:wtab,tarray:tarray,earray:earray, $
    ty:ty,en:en,bkg_type:ptr_new(bkg_type),bkg_button:bkg_button,bkg_mode:ptr_new(bkg_mode), $
    bkg_str:ptr_new(bkg_str),plotman_obj:plotman_obj,bkg_obj:bkg_obj,fit_button:fit_button,fit_mode:ptr_new(fit_mode),$
    fit_tr:ptr_new(fit_tr),fit_times:fit_times}
  ;,bkg_data:bkg_data
  widget_control,w_base,set_uvalue = state
  widget_control,/hourglass

  xmanager, 'bkg_widget', w_base, group_leader=group ;,/no_block
  
  ;stop
  info_data=*state.info_data
  bkg_mode = *state.bkg_mode
  bkg_str = *state.bkg_str

end
