;+
; Mission     : ASO-S/HXI
;   
; NAME:
;     COUNTWIDGET
;   
; Puropse:
;     Display the count of 91 modulation detectors within the time and energy range 
;     selected by the user.
;
; CALLS:
;     HXI_EN2STR
;
; INPUTS:
;     TARRAY - time array in usual format, [2,n] string
;     EARRAY - energy bins array, [2,m] float
;     COUNTs_STRUCT - struct containing det data from different time periods, [91,ntimes,nebins] 
;     COUNTS_BG_STRUCT - struct containing bkg data from different time periods, [91,ntimes,nebins]
;     EXPTS_STRUCT - struct containing exposure time in different time periods, [ntimes]
;     GROUP_LEADER - parent ID
;     
; OPTIONAL INPUTS:
;    
;
;History:
;     Written by Chang-xue Chen, 2023/03/24
;     2023/03/30, Chang-xue Chen, set default charsize based on different sys.
;     2023/05/10, Chang-xue Chen, major changes. Change the input and add display of time-series bkg/det counts
;     2023/06/13, Chang-xue Chen, add the display of spatial distribution of incident photons
;     2023/10/22, Changxue Chen, resize the window and number non-modulated detectors
;     2023/11/16, Changxue Chen, adjust the position of the incident photon distribution image
;     2023/12/04, Changxue Chen, adjust the window size
pro countwidget_event, event

  widget_control,event.top,get_uvalue = state
  widget_control,event.id,get_uvalue = uvalue
  
  case uvalue of
      'NEXT':begin
        *state.n_img = *state.n_img+1 <(state.tot_img-1)
      end

      'PREVIOUS':begin
        *state.n_img = *state.n_img-1 >0
      end

      'N_IMG':begin
        widget_control,state.text1,get_value = nimg
        *state.n_img =(nimg-1) <(state.tot_img-1) >0
      end
      
      'N_DET':begin
        widget_control,state.det_text,get_value = ndet
        *state.n_det =(ndet-1) <(91-1) >0
      end
      else:
  endcase
  
  if uvalue eq 'counts_type' then begin
    widget_control,state.wbutton,get_value = c_type ;0 / 1 / 2
    state.counts_type = c_type
    widget_control,event.top, set_uvalue=state
  endif
  
  n_e =*state.n_img  mod (state.e_n) & n_t = *state.n_img/(state.e_n)
  case state.counts_type of
    0:begin

      widget_control,state.w_draw,get_value = window1
      wset,window1
      loadct,0
      ;-------------------------------
      count =(total(state.counts_struct.(n_t),2))[*,n_e]
      count_bg = (total(state.counts_bg_struct.(n_t),2))[*,n_e]

      plot,indgen(91)+1,count,psym=10,xst=1,xtitle = 'Detector number',ytitle = 'Counts',charsize=state.char $
        ,title =hxi_en2str((state.t_str)[*,n_t],/tim) +', energy in '+(state.en_str)[0,n_e]+' to '+(state.en_str)[1,n_e] +' keV'

      oplot,indgen(91)+1,count_bg,lines = 2,psym=10
      plots,[0.75,0.79],[0.88,0.88],lines = 2,/normal
      xyouts,0.8,0.872,'BKG counts',/normal,charsize =state.char
      widget_control,state.text1,set_value = trim(*state.n_img+1)
      end
    2:begin
      widget_control,state.w_draw,get_value = window1
      wset,window1
      loadct,0
      ;-------------
      count_bg = (reform((state.counts_bg_struct.(n_t))[0,*,*]))[*,n_e]
      count_det = (reform((state.counts_struct.(n_t))[*state.n_det,*,*]))[*,n_e]
      dmax =  max(count_det)*1.1 &  dmin = min(count_bg)*0.8
      expts = state.expts_struct.(n_t)
      xx = make_array(n_elements(expts))
      for i=1,n_elements(expts)-1 do xx[i] = xx[i-1]+expts[i-1]
        
      plot,xx,count_bg,xtitle = 'Times from beginning (s)',ytitle = 'Counts',charsize=state.char ,yrange = [dmin,dmax]$
        ,title =hxi_en2str((state.t_str)[*,n_t],/tim) +', energy in '+(state.en_str)[0,n_e]+' to '+(state.en_str)[1,n_e] +' keV'$
        ,xst=1,linest =2
      
      plots,[0.75,0.79],[0.88,0.88],lines = 2,/normal
      xyouts,0.8,0.872,'BKG counts',/normal,charsize =state.char
      
      oplot,xx,count_det,lines = 0
      plots,[0.75,0.79],[0.84,0.84],lines = 0,/normal
      xyouts,0.8,0.84-0.008,'Det_'+trim(*state.n_det+1) +' counts',/normal,charsize =state.char
      
      widget_control,state.text1,set_value = trim(*state.n_img+1)
      
      end
    1:begin
      count =(total(state.counts_struct.(n_t),2))[*,n_e]
      result = hxi_photo_dis(count)
      ph_dis = reverse(reform(result[*,*,0]),2)
      det_num = reverse(reform(result[*,*,1]),2)
      dmax = 1.02*max(ph_dis); & dmin = 0.9*min(ph_dis[where(ph_dis gt 0)])
      dmin = 0.05*dmax
      
;      min_size = min([state.xsize,state.ysize],ind)
;      if ind eq 0 then begin
;        xst = 0.1& dx = 0.75  & dy = state.xsize*dx/state.ysize*9/11& yst = 0.5-dy/2
;        position = [xst,yst,xst+dx,dy+yst]
;        colorbar_position= [position[2]+0.03,position[1],position[2]+0.06,position[3]] 
;      endif else begin
      yst = 0.1& dy=0.79 & dx = state.ysize*dy/state.xsize*11/9& xst = 0.5-dx/2.
      position = [xst,yst,xst+dx,dy+yst]
      colorbar_position= [position[2]+0.03,position[1],position[2]+0.06,position[3]]
   ;   endelse
      loadct,39,/silent
      plot_image,ph_dis,max = dmax,min = dmin,position = position,charsize = 1.2,$
        title=hxi_en2str((state.t_str)[*,n_t],/tim) +', energy in '+(state.en_str)[0,n_e]+' to '+(state.en_str)[1,n_e] +' keV'
      for i = 0,10 do begin
        for j =0,8 do begin
          color =255
          if ph_dis[i,j] gt dmin+(dmax-dmin)/2 then color = 0
          if det_num[i,j] lt 91 then begin
            xyouts,i-0.2,j-0.1,'D'+trim(det_num[i,j]+1,'(I2.2)'),color = color,charsize=1.2
          endif
        endfor
      endfor
      char_def = 1.2
      xyouts,0-0.2,4+0.1,'FG',COLOR=255,CHARSIZE=char_def
      xyouts,0-0.2,4-0.3,'D98',COLOR=255,CHARSIZE=char_def
      xyouts,10-0.2,4+0.1,'FG',COLOR=255,CHARSIZE=char_def
       xyouts,10-0.2,4-0.3,'D97',COLOR=255,CHARSIZE=char_def
      xyouts,4-0.4,4+0.1,'THICK',COLOR=255,CHARSIZE=char_def
       xyouts,4-0.4,4-0.3,'D93',COLOR=255,CHARSIZE=char_def
      xyouts,5-0.2,4+0.1,'DM',COLOR=255,CHARSIZE=char_def
      xyouts,5-0.2,4-0.3,'D96',COLOR=255,CHARSIZE=char_def
      xyouts,6-0.2,4+0.1,'SA',COLOR=255,CHARSIZE=char_def
      xyouts,6-0.2,4-0.3,'D95',COLOR=255,CHARSIZE=char_def
      xyouts,7-0.4,4+0.1,'THICK',COLOR=255,CHARSIZE=char_def
      xyouts,7-0.4,4-0.3,'D92',COLOR=255,CHARSIZE=char_def
      xyouts,4-0.3,0+0.1,'THIN',COLOR=255,CHARSIZE=char_def
      xyouts,4-0.3,0-0.3,'D94',COLOR=255,CHARSIZE=char_def
      xyouts,5-0.2,0+0.1,'FG',COLOR=255,CHARSIZE=char_def
      xyouts,5-0.2,0-0.3,'D99',COLOR=255,CHARSIZE=char_def
      colorbar, /right,/vertical,max=dmax,min=dmin,format='(I)',charsize = 1.2 $
        ,position =colorbar_position
      loadct,0,/silent
      widget_control,state.text1,set_value = trim(*state.n_img+1)
     ; stop
      end
     else:
  endcase
  
  if uvalue eq 'QUIT' then widget_control,event.top,/destroy

end


pro countwidget,tarray,earray,counts_struct,counts_bg_struct,expts_struct,group_leader = group_leader
  
  if not have_windows() then begin
    message,'widgets are unavailable'
    return
  endif
  ;default setting-----------
  ;stop
  chars = os_family() eq 'unix' ? 1.3 : 1
  t_n = (size(tarray))[2]& e_n =( size(earray))[2]
  if (size(tarray))[0] eq 1 then t_n=1
  if (size(earray))[0] eq 1 then e_n=1
  tot_img = t_n*e_n &  n_img = 0 & n_det=58
  en_str = trim(earray) & t_str = atime(tarray)
  
  device,get_screen_size=sc
  fxpad = 0.02*sc(0)
  ys =  0.6*sc(1)  & xs =  1.6*ys
  xsize =  0.5*sc(0)<xs
  ysize = xsize/1.6;  20231204
  ;--------------------
  btitle = ' WIDGET DETECTOR-COUNTS ' 
  modal=1
  w_base = widget_base(TITLE = btitle, /colum,/frame, modal=modal, group_leader=group_leader )
  base2 = widget_base(w_base,/colum)
  ;-----------
  ;switch_b = 1 ;default display counts of diff dets
  base3 = widget_base(base2,/row,/align_center) ;
  type_list = ['Counts of different detectors','Spatial distribution of photons','Counts vs Time about BKG and DET: ']
  wbutton = cw_bgroup(base3,type_list,/row,/exclusive,uvalue='counts_type',space=0.5*fxpad)
  WIDGET_CONTROL, wbutton, set_value=0
  
  det_text = widget_text(base3,value = trim(n_det+1),uvalue = 'N_DET',/editable,xsize= 0.12*fxpad)
  ;----------------
  base4 = widget_base(base2,/column,/align_cent )
  draw1 = widget_draw(base4,xsize = xsize ,ysize =ysize)
  ;------------------
  base5 = widget_base(base2, /row, /align_cent,space=0.6*fxpad) ;, xpad=7*fxpad, space=5.*fspace, ypad=0.6*fypad
  pre = widget_button(base5,value='         PREVIOUS      ', $
    uvalue='PREVIOUS', /no_release)
  text1 = widget_text(base5,value = '1',uvalue = 'N_IMG',/editable,xsize= 0.12*fxpad)
  tmp = widget_label(base5,value = '/ '+trim(tot_img))
  next = widget_button(base5,value='        NEXT        ',uvalue='NEXT', $
    /no_release)
  quit = widget_button(base5,value='        QUIT        ',uvalue='QUIT', $
    /no_release)

  widget_control,w_base,/realize
  widget_control,draw1,get_value = window1
  wset,window1
  ;-rebin data and set default (plot det counts)---------------------------
  count0 = reform((total(counts_struct.(0),2))[*,0])
  count0_bg = reform((total(counts_bg_struct.(0),2))[*,0])
  loadct,0
  ;--------------------------------------
  plot,indgen(91)+1,count0,psym=10,xst=1,xtitle = 'Detector number',ytitle = 'Counts',charsize=chars $
    ,title =hxi_en2str(t_str[*,0],/tim) +', energy in '+en_str[0,0]+' to '+en_str[1,0] +' keV'
  
  oplot,indgen(91)+1,count0_bg,lines = 2,psym=10
  plots,[0.75,0.79],[0.88,0.88],lines = 2,/normal
  xyouts,0.8,0.872,'BKG counts',/normal,charsize = chars
  
  state = {w_base:w_base,xsize:xsize,ysize:ysize,n_img :ptr_new(n_img),tot_img:tot_img,$
    counts_struct:counts_struct,counts_bg_struct:counts_bg_struct,counts_type:0, $ ;cxhcen 0510
    expts_struct:expts_struct,en_str:en_str,t_str:t_str,t_n:t_n,e_n:e_n,$
    w_draw:draw1,text1:text1,char:chars,wbutton:wbutton,det_text:det_text,n_det:ptr_new(n_det)}
  
  widget_control,w_base,set_uvalue = state
  
  xmanager, 'countwidget', w_base, group_leader=group,/no_block

end

