;+
; Mission : ASO-S/HXI
; Puropse:
;  Generate composite components that enable the user to select collimators (for imaging)
;
; Calls:
;  detwidget.pro
;  
; History:
;  Written by Fanxiaoyu Xia, 2021/11/10
;  2021/09/21, Fanxiaoyu Xia, updated the parameters immediately after modification, if the values
;   are out of range or ' ', then return error message and modified them back to default values.
;  2023/03/30, Chang-xue Chen, delete 'Sub-collimator Group' component, add 'Select sub-collimators' component.
;   Users can select specific detectors from dialog box via detwidget.pro.

function hxi_coll_widget3_event, event
  base = event.handler
  stash = widget_info(base, /child)
  widget_control, stash, get_uvalue=state
  widget_control, event.id, get_uvalue=uvalue

  if strlen(uvalue) le 2 then begin
    num=fix(uvalue)
    if num le 2 then begin
      widget_control,state.text0,get_value=text0
      widget_control,state.text1,get_value=text1

      ;Ensure the input value is between 1-91 or empty, before<=after
      if text0 ne '' and text1 ne '' then begin
        if text0 eq ' ' or text1 eq ' ' then begin
          a = dialog_message('Detector range is null.')
          if (num eq 1 and text1 eq ' ') then begin
            widget_control,state.text1,set_value=state.cdet_range[1]
            state.det_range[1]=state.cdet_range[1]
          endif
          if (num eq 1 and text0 eq ' ') then begin
            widget_control,state.text0,set_value=state.cdet_range[0]
            state.det_range[0]=state.cdet_range[0]
          endif
          if (num eq 2 and text0 eq ' ') then begin
            widget_control,state.text0,set_value=state.cdet_range[0]
            state.det_range[0]=state.cdet_range[0]
          endif
          if (num eq 2 and text1 eq ' ') then begin
            widget_control,state.text1,set_value=state.cdet_range[1]
            state.det_range[1]=state.cdet_range[1]
          endif
        endif else begin
          ftext0=fix(text0)
          ftext1=fix(text1)
          if (ftext0 lt 1) or (ftext0 gt 91) or (ftext1 lt 1) or (ftext1 gt 91) $
            then begin  
            a = dialog_message('Out of range [1-91].')  ;230512, [0-90]->[1-91],Yang Su
            if uvalue eq '1' then widget_control,state.text0,set_value='1'
            if uvalue eq '2' then widget_control,state.text1,set_value='91'
          endif
          state.det_range[0]=text0
          state.det_range[1]=text1
          det_id= hxi_get_detids(det_range=fix(state.det_range)-1)
          detid = replicate(0,91) & detid[det_id]=1
          *state.cdetid = detid
        endelse
      endif else begin
        state.det_range=['','']
      endelse
    endif
    widget_control, stash, set_uvalue=state
  endif
  
  if uvalue eq 'sel_sub_coll' then begin
    
    if state.det_range[0] ne '' then begin
      det_id= hxi_get_detids(det_range=fix(state.det_range)-1)
      detid = replicate(0,91)
      detid[det_id] = 1
    endif else begin
      detid = *state.cdetid
    endelse
    detwidget,detid,group_leader=event.top
    *state.cdetid = detid
   
    state.det_range= ['','']
    widget_control,state.text0,set_value=''
    widget_control,state.text1,set_value=''
    widget_control,stash, set_uvalue=state
    
  endif
  ret = {id: base, top: event.top, handler: 0L}
  return,ret
end

function hxi_coll_widget3, parent, uvalue=uvalue

  w_base = widget_base( parent, $
    event_func='hxi_coll_widget3_event', /base_align_left, $
    ;/column, uvalue=uvalue, y_scroll_size=150, $
    /row, uvalue=uvalue, $
    title='Collimator/Detector Options');/frame)

  w_top = widget_base(w_base, /row)
  w_collbase = widget_base(w_top, /row, /align_left, space=5,ypad = 1)
  w_labels = widget_label( w_collbase, value='Select Detectors:  By range', /align_left) ;230513,YS
  w_seg = widget_base(w_collbase, /row )
  cdet_range=['19','91']
  text0=widget_text(w_seg,value=cdet_range[0], xsize=3, /editable,/all_events,uvalue='1')
  lab1=widget_label(w_seg, value=' to ')
  text1=widget_text(w_seg,value=cdet_range[1], xsize=3, /editable,/all_events,uvalue='2')

  lab2=widget_label(w_collbase, value=' or ',/align_left)
  tmp = widget_button( w_collbase, value='By Groups', uvalue='sel_sub_coll' );230513,YS

  defaultdetid = replicate(0,91)
  state= { w_base: w_base, text0:text0,text1:text1,cdet_range:cdet_range,$
    det_range:cdet_range, cdetid:ptr_new(defaultdetid)}
  widget_control, widget_info(w_base, /child ), set_uvalue=state

  return, w_base
end