;+
; Mission     : ASO-S/HXI
; Name: hxi_draw_flags
; 
; Calls:
;   mark_intervals.pro
;   
; Purpose: 
;   Plotting hxi flags on quicklook plots.
;
; Input arguments:
;   sbar - Start times of flags, in anytime()
;   ebar - End times of flags, the same dimension as sbar
;   flag_ind - The index of the flag needed drawn
;   
; Keywords:
;   n_bar - The number of bars drawn in the window, it can be used as input/output.
;    When setting multi flags, the corresponding bar position will be automatically adjusted
;   utplot - X-axis in UT time
;   
;Note: This pro can only draw one label at a time.
;   (The flags info is from quicklook data)
;   
; History:
;   Written by Chang-xue Chen,2022/08/18

pro hxi_draw_flags,sbar=sbar,ebar=ebar,flag_ind=flag_ind,n_bar =n_bar,utplot=utplot

  flag_names = ['SAF','NT','SAA','CAL','BM','FL']  ;230601, YS
  flag_colors = [253, 250, 245, 258, 243, 239] 
  flag_line = [0,1,1,0,0,0]
  
  if not keyword_set(n_bar) then n_bar =0

  mark_intervals,sbar = sbar,ebar=ebar,label = flag_names[flag_ind] $
    ,color = flag_colors[flag_ind],n_bar=n_bar,utplot = utplot $
    ,longline = flag_line[flag_ind]
  
end



